/***************************************************************************
                          CallTab.cpp  -  description
                             -------------------
    begin                : Tue Jan 15 2002
    copyright            : (C) 2002 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "CallTab.h"

CallTab::CallTab(QWidget *parent):QFrame(parent)
{
	grid = new QGridLayout(this,2,1);
 	table = new Table(0,5,this);
 	table->setSelectionMode(QTable::NoSelection);
 	table->setLeftMargin(0);
 	col = table->horizontalHeader();
 	table->verticalHeader()->hide();
 	col->setLabel(0,i18n("Date"));
 	col->resizeSection(0,90);
 	col->setLabel(1,i18n("Time"));
 	col->resizeSection(1,90);
	col->setLabel(2,i18n("Caller Name"));
	col->resizeSection(2,170);
 	col->setLabel(3,i18n("Caller Number"));
 	col->resizeSection(2,100);
 	col->setLabel(4,i18n("Called Number"));
 	col->resizeSection(4,100);
 	col->setClickEnabled(false);
 	table->setColumnStretchable(2,true);
 	table->setColumnStretchable(3,true);
//	table->setColumnStretchable(4,true);
 	
 	buttons = new QFrame(this);
 	buttonGrid = new QGridLayout(buttons,1,1);
 	resetButton = new QPushButton(i18n("Reset"),buttons);
 	buttonGrid->addWidget(resetButton,0,0,Qt::AlignCenter);
 	grid->addWidget(table,0,0);
 	grid->addWidget(buttons,1,0);
 	grid->setRowStretch(0,100);
	grid->setRowStretch(1,20);
 	
 	
 	connect(table,SIGNAL(clicked(int,int,int,const QPoint&)),SLOT(clicked(int)));
	connect(table,SIGNAL(valueChanged(int,int)),SLOT(changed(int,int)));
 	connect(resetButton,SIGNAL(clicked()),SLOT(resetTelmond()));
}

CallTab::~CallTab()
{
}

void CallTab::clear()
{
	table->setNumRows(0);
}

void CallTab::addRow(QString date,QString time,QString caller,QString source,QString aim)
{
	int row = table->numRows();
	table->setNumRows(row+1);
	
	// move all entries one position down
	for(int i=row;i>0;i--)
	{
		table->swapRows(i,i-1);		
	}
	
 	table->setText(0,0,date);
 	table->setText(0,1,time);
	table->setText(0,2,caller);
 	table->setText(0,3,source);
 	table->setText(0,4,aim);
 	
 	table->hide();
 	table->show();	
}

void CallTab::clicked(int row)
{
	QTableSelection selection;
	selection.init(row,0);
	selection.expandTo(row,4);
	table->addSelection(selection);
}

void CallTab::changed(int row, int col)
{
	if (col == 2)
	{
		QString caller = table->text(row,col);
		QString callerid = table->text(row,col+1);
		KSimpleConfig *config = new KSimpleConfig("TrayImonc");
		QStringList phoneBook = config->readListEntry("PhoneBook");
		QStringList match = phoneBook.grep(callerid);
		QStringList newPhoneBook;
		for (QStringList::Iterator it = phoneBook.begin(); it != phoneBook.end(); it++ )
		{
		  if (*it != match.join(""))
		    newPhoneBook.append(*it);
		}
		newPhoneBook.append(QString(callerid + ":" + caller));
		newPhoneBook.sort();
		config->writeEntry("PhoneBook",newPhoneBook);
		config->sync();
		emit updatePhoneBook();
		
	}
}

void CallTab::setSize(int width, int height)
{
	table->resize(width,height);
}

void CallTab::resetTelmond()
{
 	emit reset();
 	
 	//Diese Sache sollte noch abgesichert werden
 	//die Tabelle erst leeren, wenn das Reset auf dem Router
 	//auch erfolgreich ausgefhrt wurde
 	table->setNumRows(0);
}

