//
// C++ Implementation: ConnTab
//
// Description: 
//
//
// Author: Thomas Bosch <tbosch@gmx.de>, (C) 2005
//
// Copyright: See COPYING file that comes with this distribution
//
//
#include "ConnTab.h"
#include <qfile.h>
#include <qmessagebox.h>
#include <qapp.h>
#include <qfiledialog.h>
#include <qdialog.h>
#include <qlayout.h>

/* -------------------------------------------------------------------
 * constructor function
 * -------------------------------------------------------------------*/
ConnTab::ConnTab(QWidget *parent):QFrame(parent)
{
	grid = new QGridLayout(this,2,1);
 	table = new Table(0,15,this);
 	table->setSelectionMode(QTable::NoSelection);
 	table->setLeftMargin(0);
 	col = table->horizontalHeader();
 	table->verticalHeader()->hide();
 	col->setLabel(0,i18n("Provider"));
 	col->resizeSection(0,120);
 	col->setLabel(1,i18n("Date (connected)"));
 	col->resizeSection(1,80);
	col->setLabel(2,i18n("Time (connected)"));
	col->resizeSection(2,60);
 	col->setLabel(3,i18n("Date (disconnected)"));
 	col->resizeSection(3,80);
 	col->setLabel(4,i18n("Time (disconnected)"));
 	col->resizeSection(4,60);
 	col->setLabel(5,i18n("Seconds"));
 	col->resizeSection(5,50);
 	col->setLabel(6,i18n("Seconds (effective)"));
 	col->resizeSection(6,50);
 	col->setLabel(7,i18n("Costs"));
 	col->resizeSection(7,50);
 	col->setLabel(8,i18n("-"));
 	col->resizeSection(8,15);
 	col->setLabel(9,i18n("Incoming traffic"));
 	col->resizeSection(9,80);
 	col->setLabel(10,i18n("-"));
 	col->resizeSection(10,15);
 	col->setLabel(11,i18n("Outgoing traffic"));
 	col->resizeSection(11,80);
 	col->setLabel(12,i18n("Device"));
 	col->resizeSection(12,50);
 	col->setLabel(13,i18n("Seconds per unit"));
 	col->resizeSection(13,30);
 	col->setLabel(14,i18n("Costs per unit"));
 	col->resizeSection(14,50);
 	col->setClickEnabled(false);
 	table->setColumnStretchable(14,true);
	
	ask = 0;
	
 	buttons = new QFrame(this);
 	buttonGrid = new QGridLayout(buttons,1,1);
 	exportButton = new QPushButton(i18n("CSV export"),buttons);
 	buttonGrid->addWidget(exportButton,0,0,Qt::AlignCenter);
 	grid->addWidget(table,0,0);
 	grid->addWidget(buttons,1,0);
 	grid->setRowStretch(0,100);
	grid->setRowStretch(1,20);

 	connect(exportButton,SIGNAL(clicked()),SLOT(exportCSV()));
}

/* -------------------------------------------------------------------
 * destructor and clearing function
 * -------------------------------------------------------------------*/
ConnTab::~ConnTab()
{
}

void ConnTab::clear()
{
	table->setNumRows(0);
}

/* -------------------------------------------------------------------
 * exports the whole table to one file you want
 * -------------------------------------------------------------------*/
void ConnTab::exportCSV()
{
	QFileDialog* fd = new QFileDialog( QDir::homeDirPath() );
    	fd->setMode( QFileDialog::AnyFile );
	fd->setFilter( i18n("Log files (*.log)") );
	fd->setViewMode( QFileDialog::Detail );
    	QString fileName;
    	if ( fd->exec() == QDialog::Accepted )
	{
        	fileName = fd->selectedFile();
		fileNameCSV = fileName;
		if (QFile::exists(fileName))
		{
			if (!ask)
			{
				ask = new QDialog(this);
				askLayout = new QVBoxLayout( ask, 11, 6, "askLayout");
				label = new QLabel(ask, "label");
				label->setText(i18n("This file already exits, really want to overwrite?"));
				askLayout->addWidget(label);
				spacer1 = new QSpacerItem( 20, 20, QSizePolicy::Minimum, QSizePolicy::Expanding );
				askLayout->addItem( spacer1 );
				layout = new QHBoxLayout( 0, 0, 6, "layout");
				yesButton = new QPushButton(ask, "yesButton");
				yesButton->setText(i18n("Yes"));
				layout->addWidget( yesButton );
				spacer2 = new QSpacerItem( 111, 20, QSizePolicy::Expanding, QSizePolicy::Minimum );
				layout->addItem( spacer2 );
				noButton = new QPushButton(ask, "noButton");
				noButton->setText(i18n("No"));
				layout->addWidget( noButton );
				askLayout->addLayout( layout );
				connect(yesButton, SIGNAL(clicked()), this, SLOT(overwrite()));
				connect(noButton, SIGNAL(clicked()), this, SLOT(askHide()));
			}
			ask->show();
		}
		else 
		{
		  	QStringList connCSV = convertCSV(connections);
			writeFile(connCSV, fileNameCSV);
		}
	}
}

/* ------------------------------------------------------------------
 * overwrites file
 * ----------------------------------------------------------------*/
void ConnTab::overwrite()
{
	QStringList connCSV = convertCSV(connections);
	writeFile(connCSV, fileNameCSV);
	ask->hide();
}

/* ------------------------------------------------------------------
 * hiding ask dialog
 * ----------------------------------------------------------------*/
void ConnTab::askHide()
{
	ask->hide();
}

/* -------------------------------------------------------------------
 * sets the local variable 'connections'
 * used to keep a local connection status update
 * -------------------------------------------------------------------*/
void ConnTab::setConnections(QStringList conn)
{
	connections = conn;
}

/* -------------------------------------------------------------------
 * adds a given row with all necessary values to the connection table
 * -------------------------------------------------------------------*/
void ConnTab::addRow(QString provider,QString dateIn,QString timeIn,QString dateOut,QString timeOut,
			QString seconds,QString secondsEff,QString costs,QString x,QString trafficIn,
			QString y,QString trafficOut,QString device,QString secondsUnit,QString costsUnit)
{
	int row = table->numRows();
	table->setNumRows(row+1);
	
	// move all entries one position down
	for(int i=row;i>0;i--)
	{
		table->swapRows(i,i-1);		
	}
	
 	table->setText(0,0,provider);
 	table->setText(0,1,dateIn);
	table->setText(0,2,timeIn);
 	table->setText(0,3,dateOut);
 	table->setText(0,4,timeOut); 
	table->setText(0,5,seconds);
 	table->setText(0,6,secondsEff); 
	table->setText(0,7,costs);
 	table->setText(0,8,x); 
	table->setText(0,9,trafficIn);
 	table->setText(0,10,y); 
	table->setText(0,11,trafficOut);
 	table->setText(0,12,device); 
	table->setText(0,13,secondsUnit);
 	table->setText(0,14,costsUnit);
 	
 	table->hide();
 	table->show();	
}

/* -------------------------------------------------------------------
 * writes a list to a given file
 * -------------------------------------------------------------------*/
void ConnTab::writeFile(QStringList sList, QString fileStr)
{
	QFile file(fileStr);
	if (!file.open(IO_WriteOnly)) {
		QMessageBox::critical(
		this,
		i18n("Open failed"),
		i18n("Could not open connections file for reading/writing: %1").arg( qApp->translate("QFile",file.errorString()) )
		);
		return;
	}
	
	QTextStream stream( &file );
	QString line;
	for ( QStringList::Iterator it = sList.begin(); it != sList.end(); ++it )
	{
		stream << *it << "\n";
	}
	file.close();
}

/* -------------------------------------------------------------------
 * converts a connection list to csv and
 * writes this list to a given log file (see options)
 * -------------------------------------------------------------------*/
void ConnTab::writeLogFile(QStringList sListOrig, QString fileStr)
{
	QStringList sList = convertCSV(sListOrig);
	
	QFile file(fileStr);
	if (!file.open(IO_ReadWrite | IO_Append)) {
		QMessageBox::critical(
		this,
		i18n("Open failed"),
		i18n("Could not open connections file for reading/writing: %1").arg( qApp->translate("QFile",file.errorString()) )
		);
		return;
	}
	
	QTextStream stream( &file );
	QString line;
	for ( QStringList::Iterator it = sList.begin(); it != sList.end(); ++it )
	{
		if ((line = stream.readLine()) == (QString)0)
		{
			stream << *it << "\n";
		}
		else if (line == *it)
		{
			// nothing
		}
		else if (line != *it)
		{
			it--;
		}
	}
	file.close();
}


/* -------------------------------------------------------------------
 * converts the list get from the router into a csv form, that means
 * the data cells were seperated by a semicolon
 * -------------------------------------------------------------------*/
QStringList ConnTab::convertCSV(QStringList before)
{
	QStringList after;
	QString line;
	QString csv = ";";  // trennzeichen zwischen den datenzellen
	QString temp;
	
	for ( QStringList::Iterator it = before.begin(); it != before.end(); ++it )
	{
		temp = (*it).latin1();
		
		if(temp.contains("ERR") || temp.contains("OK"))
			break;
	 	
		temp = temp.right(temp.length()-1);
		QString provider = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-21);
		QString dateIn = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-dateIn.length()-1);
		QString timeIn = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-timeIn.length()-1);
		QString dateOut = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-dateOut.length()-1);
		QString timeOut = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-timeOut.length()-1);
		QString seconds = temp.left(5);
		seconds = seconds.simplifyWhiteSpace();
		temp = temp.right(temp.length()-6);
		QString secondsEff = temp.left(5);
		secondsEff = secondsEff.simplifyWhiteSpace();
		temp = temp.right(temp.length()-6);
		QString costs = temp.left(6);
		costs = costs.simplifyWhiteSpace();
		temp = temp.right(temp.length()-7);
		QString x = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-x.length()-1);
		QString trafficIn = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-trafficIn.length()-1);
		QString y = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-y.length()-1);
		QString trafficOut = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-trafficOut.length()-1);
		QString device = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-device.length()-1);
		QString secondsUnit = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-secondsUnit.length()-1);
		QString costsUnit = temp.left(temp.find("\n"));
		
		QString yearIn = dateIn.left(dateIn.find("/"));
		dateIn = dateIn.right(dateIn.length()-yearIn.length()-1);
		QString monthIn = dateIn.left(dateIn.find("/"));
		dateIn = dateIn.right(dateIn.length()-monthIn.length()-1);
		QString dayIn = dateIn.left(dateIn.find("/"));
		dateIn = "";
		QTextStream ts( &dateIn, IO_WriteOnly );
		ts << dayIn << "." << monthIn << "." << yearIn;		
		
		QString yearOut = dateOut.left(dateOut.find("/"));
		dateOut = dateOut.right(dateOut.length()-yearOut.length()-1);
		QString monthOut = dateOut.left(dateOut.find("/"));
		dateOut = dateOut.right(dateOut.length()-monthOut.length()-1);
		QString dayOut = dateOut.left(dateOut.find("/"));
		dateOut = "";
		QTextStream ts2( &dateOut, IO_WriteOnly );
		ts2 << dayOut << "." << monthOut << "." << yearOut;
		
		line.append(provider + csv);
		line.append(dateIn + csv);
		line.append(timeIn + csv);
		line.append(dateOut + csv);
		line.append(timeOut + csv);
		line.append(seconds + csv);
		line.append(secondsEff + csv);
		line.append(costs + csv);
		line.append(x + csv);
		line.append(trafficIn + csv);
		line.append(y + csv);
		line.append(trafficOut + csv);
		line.append(device + csv);
		line.append(secondsUnit + csv);
		line.append(costsUnit);
		
		after += line;
		line = "";
	}
	
	return after;
}

#include "ConnTab.moc"
