/***************************************************************************
                          Fli4lTalk.cpp  -  description
                             -------------------
    begin                : Sun Mar 17 2002
    copyright            : (C) 2002 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Fli4lTalk.h"
#include <iostream.h>


Fli4lTalk::Fli4lTalk()
{
	fd = -1;
}

Fli4lTalk::~Fli4lTalk()
{
 	(void)close(fd);
}

void Fli4lTalk::connectToHost(QString host_name,int port)
{
    struct sockaddr_in	addr;
    struct hostent *	host_p;
    int opt = 1;
    struct timeval timeout;
    int n;
    fd_set wset;
    char    buf[256];

    (void) memset ((char *) &addr, 0, sizeof (addr));

    if ((addr.sin_addr.s_addr = inet_addr (host_name.latin1())) == INADDR_NONE)
    {
			host_p = gethostbyname (host_name.latin1());

			if (! host_p)
			{
	    	emit connectFailed();
	    	return;
			}

			(void) memcpy ((char *) (&addr.sin_addr), host_p->h_addr,host_p->h_length);
    }

    addr.sin_family  = AF_INET;
    addr.sin_port    = htons ((unsigned short) port);

    if ((fd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
    {		
			emit connectFailed();
			return;
    }

    (void) setsockopt (fd, IPPROTO_TCP, TCP_NODELAY,(char *) &opt, sizeof (opt));

    //Setzt Socket auf Non-Blocking
 		fcntl (fd, F_SETFL, O_NONBLOCK);

 		n = 0;
		if ((n = (::connect (fd, (struct sockaddr *) &addr, sizeof (addr)))) < 0)
			if (errno != EINPROGRESS)
			{
				emit connectFailed();
				return;
			}
		
		if(n==0)
		{
			emit connected();
	    return;
	  }
		
    FD_ZERO(&wset);
		FD_SET(fd, &wset);
		timeout.tv_sec = 3;     //maximal 3 Sekunden fr Verbindungsaufbau
		timeout.tv_usec = 0;

   	if(select(fd+1,NULL,&wset,NULL,&timeout)==0)
    {
    	//Connect timeout
    	(void) close (fd);
			emit connectFailed();
			return;
    }

    //hier wird berprft, ob auf das socket wirklich geschrieben werden kann
    //In einigen Fllen sagt select nmlich, dass auf dem Socket geschrieben werden kann,
    //obwohl connection refused, doch weder errno sagt etwas, noch der Rckgabewert
    if(FD_ISSET(fd,&wset))
    {
      if (FD_ISSET(fd, &wset))
      {
        int d = write(fd, "o\r\n", 3);
        if (d < 0)
        {
          (void) close (fd);
			    emit connectFailed();
          return;
        }
      }
    }	

    fcntl (fd, F_SETFL, O_RDWR);

    //nach dem vorherigen write muss nochmal vom Socket gelesen werden,
    //weil sonst mit getAnswer eine Response zurckkommt der nicht zum
    //aktuellen Kommando passt
    //das bringt z.B. Probleme bei der Passwortabfrage
    read(fd,buf,8192);

    emit connected();
	  return;
}

void Fli4lTalk::disconnect()
{
	(void) close (fd);
    return;
}

void Fli4lTalk::sendCommand(QString command)
{
	char    buf[256];
 	int	    len = strlen (command.latin1());

  sprintf (buf, "%s\r\n", command.latin1());

  write (fd, buf, len + 2);
}

QString Fli4lTalk::getAnswer()
{
		static char buf[8192];
    int		len;
    fd_set rset;
    struct timeval timeout;

		FD_ZERO(&rset);
		FD_SET(fd, &rset);
		timeout.tv_sec = 3;     //maximal 3 Sekunden fr read
		timeout.tv_usec = 0;
		
		//Test,ob Daten verfgbar sind
		//dies dient zur Erkennung von Lost Connections
		select(fd+1,&rset,NULL,NULL,&timeout);
		
		if(FD_ISSET(fd,&rset))
			len = read(fd,buf,8192);	
		else
		{
			//cout << "lost connection" << endl;
			emit lostConnection();
			buf[0] = '\0';
			return NULL;
		}
		
    while (len > 1 && (buf[len - 1] == '\n' || buf[len - 1] == '\r'))
			{
				buf[len - 1] = '\0';
				len--;
			}
		
    if (! strncmp (buf, "OK ", 3))			/* OK xxxx	    */
			{
				return ((char*)buf + 3);
			}
    else if (len > 2 && ! strcmp (buf + len - 2, "OK"))
			{
				*(buf + len - 2) = '\0';
				return ((char*)buf);
			}
    else if (len == 2 && ! strcmp (buf + len - 2, "OK"))
			{
				return ((char*)buf);
			}
		
		buf[0] = '\0';
    return ((char*)buf);	
}


QStringList Fli4lTalk::getList(QString command)
{
	QStringList sList;
  QString temp(getBuf(command));
	sList = QStringList::split("\n",temp);

	return sList;
}

QString Fli4lTalk::getBuf(QString command)
{
	char wbuf[64];
	int len = strlen(command.latin1());
	static char *   buf;
	static int      buffer_size = 0;
	char            ch[1];
	int             n;
	int             at_bol = 1;             /* flag: at beginning of line   */
	int             last_line = 0;          /* flag: last line              */
	
	sprintf (wbuf, "%s\r\n", command.latin1());
	write (fd, wbuf, len + 2);

	if (buffer_size == 0)
		{
			buffer_size = 8192;
			buf = (char*)malloc (buffer_size);
		}
	
	for (n = 0; read (fd, ch, 1) > 0; n++)
    {
			if (ch[0] == '\r')
				{
					n--;
					continue;
				}

        if (n + 1 == buffer_size)
					{
            buffer_size += 8192;
            buf = (char*)realloc (buf, buffer_size);
					}

        buf[n] = ch[0];
				if (at_bol && ch[0] != ' ')
					{
						last_line = 1;
						at_bol = 0;
					}
        else if (ch[0] == '\n')
					{
            at_bol = 1;
						
            if (last_line == 1)
							{
                break;
							}
					}
        else
					{
						at_bol = 0;
					}
    }
	
	buf[n] = '\0';

	return ((char*)buf);
}
