/***************************************************************************
                          Menu.cpp  -  description
                             -------------------
    begin                : Sun Oct 21 2001
    copyright            : (C) 2001 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Menu.h"

Menu::Menu(QWidget *parent):KPopupMenu(parent)
{
	insertTitle(SmallIcon("trayimonc"),"TrayImonc");
	config = new KSimpleConfig("TrayImonc");
	showtraffic = config->readBoolEntry(i18n("Trafficinfo"));
	showcalls = config->readBoolEntry("ShowCalls");
	default_circuit = new QPopupMenu();
	default_circuit->insertItem(i18n("Automatic (LCR)"),this,SIGNAL(setDefaultCircuit(int)),0,0,-1);
	default_circuit->setItemChecked(0,true);
	default_circuit->adjustSize();
	if (config->readBoolEntry("MenuState",true))
		insertItem(i18n("State"),this,SIGNAL(showState()),0,17,-1);
	if (config->readBoolEntry("MenuDefaultRoute",true))
		insertItem(i18n("Default-Route"),default_circuit,15,-1);
	if (config->readBoolEntry("MenuDial",true))
		insertItem(i18n("Dial"),this,SIGNAL(dial()),0,0,-1);
	if (config->readBoolEntry("MenuAddLink",true))
	{
		insertItem(i18n("AddLink"),this,SIGNAL(addlink()),0,1,-1);
		setItemEnabled(1,false);
	}
	if (config->readBoolEntry("MenuReboot",true))
		insertItem(i18n("Reboot"),this,SLOT(reboot()),0,2,-1);
	insertSeparator(7);
	if (config->readBoolEntry("MenuAuto",true))
		insertItem(i18n("Auto"),this,SLOT(changedDialMode(int)),0,5,-1);
	if (config->readBoolEntry("MenuManual",true))
		insertItem(i18n("Manual"),this,SLOT(changedDialMode(int)),0,6,-1);
	if (config->readBoolEntry("MenuOff",true))
	{
		insertItem(i18n("Off"),this,SLOT(changedDialMode(int)),0,7,-1);
		insertSeparator(11);
	}
	if (config->readBoolEntry("MenuCheckMail",true))
		insertItem(i18n("Check Mail"),this,SIGNAL(check_mail()),0,15,-1);
	if (config->readBoolEntry("MenuShowCalls",true))
	{
		insertItem(i18n("Show Incoming Calls"),this,SLOT(show_calls()),0,9,-1);
		setItemChecked(9,showcalls);
	}
	if (config->readBoolEntry("MenuTrafficMonitor",true))
	{
		insertItem(i18n("Show TrafficMonitor"),this,SLOT(traffic_info()),0,10,-1);
		setItemChecked(10,showtraffic);
		insertSeparator(14);
	}
	
	KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
	if (config->readBoolEntry("MenuOptions",true))
		insertItem(SmallIcon("configure"),i18n("Options"),this,SIGNAL(options()),0,12,-1);
	else if (args->isSet("cfg"))
		insertItem(SmallIcon("configure"),i18n("(Options)"),this,SIGNAL(options()),0,12,-1);
	
	if (config->readBoolEntry("MenuHelp",true))
		insertItem(SmallIcon("help"),i18n("Help"),this,SIGNAL(help()),0,13,-1);
	
	insertItem(SmallIcon("trayimonc"),i18n("About TrayImonc"),this,SIGNAL(about()),0,14,-1);
	insertSeparator(18);
	if (config->readBoolEntry("MenuExit",true))
		insertItem(SmallIcon("exit"),i18n("Exit"),this,SIGNAL(exitSig()),0,16,-1);
	else if (args->isSet("cfg"))
		insertItem(SmallIcon("exit"),i18n("(Exit)"),this,SIGNAL(exitSig()),0,16,-1);
	adjustSize();
}

Menu::~Menu()
{

}

void Menu::connected()
{
	emit show_calls(showcalls);
	emit trafficinfo(showtraffic);
}

void Menu::showSlot(QPoint point)
{
	exec(point);
}

void Menu::addCircuit(QString circuit)
{
	default_circuit->insertItem(circuit,this,SIGNAL(setDefaultCircuit(int)),0,default_circuit->count(),-1);
	default_circuit->adjustSize();
}

void Menu::setDefaultRoute(int num)
{
	for(int i = 0;i<(int)default_circuit->count();i++)
	{
		if(num == i)
		{
			default_circuit->setItemChecked(i,true);
		}
		else
		{
			default_circuit->setItemChecked(i,false);
		}
	}
}

void Menu::dialed()
{
	changeItem(0,i18n("Hangup"));
}

void Menu::hangup()
{
	changeItem(0,i18n("Dial"));
}

void Menu::isdnDialed()
{
	changeItem(0,i18n("Hangup"));
	setItemEnabled(1,true);
}

void Menu::isdnHangup()
{
	changeItem(0,i18n("Dial"));
	setItemEnabled(1,false);
}

void Menu::addedLink()
{
	changeItem(1,i18n("Remove Link"));
}

void Menu::removedLink()
{
	changeItem(1,i18n("Add Link"));
}

void Menu::changedDialMode(int id)
{
	switch(id)
	{
	 	case 5: emit dialMode("auto");break;
	 	case 6: emit dialMode("manual");break;
	 	case 7: emit dialMode("off");break;
	}
}

void Menu::dialAuto()
{
	setItemChecked(5,true);
	setItemChecked(6,false);
	setItemChecked(7,false);
}

void Menu::dialManual()
{
	setItemChecked(5,false);
	setItemChecked(6,true);
	setItemChecked(7,false);
}

void Menu::dialOff()
{
	setItemChecked(5,false);
	setItemChecked(6,false);
	setItemChecked(7,true);
}

void Menu::show_calls()
{
	if(showcalls)
	{
		showcalls = false;
		setItemChecked(9,false);
		emit show_calls(false);
	}
	else
	{
		showcalls = true;
		setItemChecked(9,true);
		emit show_calls(true);
	}
	config->writeEntry("ShowCalls",showcalls);
	config->sync();
}

void Menu::traffic_info()
{
	if(showtraffic)
	{
		showtraffic = false;
		setItemChecked(10,false);
		emit trafficinfo(false);
	}
	else
	{
		showtraffic = true;
		setItemChecked(10,true);
		emit trafficinfo(true);
	}
	config->writeEntry("Trafficinfo",showtraffic);
	config->sync();
}

void Menu::halt()
{
	dialog = new QDialog(0,i18n("Halt"));
  dialog->setCaption(i18n("TrayImonc -- Halt"));
  QGridLayout *grid = new QGridLayout(dialog, 4, 2 );
  QLabel *label = new QLabel(dialog);
  label->setText(i18n("Do you really want to halt the fli4l-router?"));

  QHBox *buttons = new QHBox(dialog);
	QButton *ok = new QPushButton(i18n("Yes"),buttons);
	connect(ok,SIGNAL(clicked()),this,SLOT(doHalt()));
	QButton *cancel = new QPushButton(i18n("No"),buttons);
	connect(cancel,SIGNAL(clicked()),this,SLOT(dialogCancel()));
	
  grid->addMultiCellWidget(label, 0,0,0,1,Qt::AlignCenter);
	
  grid->addMultiCellWidget(buttons,1,1,0,1,Qt::AlignCenter);
	grid->addRowSpacing(0,40);
	grid->addColSpacing(0,300);
	grid->addRowSpacing(1,40);
	
  dialog->show();
}

void Menu::reboot()
{
	dialog = new QDialog(0,i18n("Reboot"));
  dialog->setCaption(i18n("TrayImonc -- Reboot"));
  QGridLayout *grid = new QGridLayout(dialog, 4, 2 );
  QLabel *label = new QLabel(dialog);
  label->setText(i18n("Do you really want to reboot the fli4l-router?"));

  QHBox *buttons = new QHBox(dialog);
	QButton *ok = new QPushButton(i18n("Yes"),buttons);
	connect(ok,SIGNAL(clicked()),this,SLOT(doReboot()));
	QButton *cancel = new QPushButton(i18n("No"),buttons);
	connect(cancel,SIGNAL(clicked()),this,SLOT(dialogCancel()));
	
  grid->addMultiCellWidget(label, 0,0,0,1,Qt::AlignCenter);
	
  grid->addMultiCellWidget(buttons,1,1,0,1,Qt::AlignCenter);
	grid->addRowSpacing(0,40);
	grid->addColSpacing(0,300);
	grid->addRowSpacing(1,40);
	
  dialog->show();
}

void Menu::doReboot()
{
	dialog->close();
	delete(dialog);
	dialog = NULL;
	emit rebootSig();
}

void Menu::doHalt()
{
	dialog->close();
	delete(dialog);
	dialog = NULL;
	emit haltSig();
}

void Menu::dialogCancel()
{
	dialog->close();
	delete(dialog);
	dialog = NULL;
}

void Menu::noISDN()
{
	setItemChecked(9,false);
	setItemEnabled(9,false);
	showcalls = false;
}


