/***************************************************************************
                          Monitor.cpp  -  description
                             -------------------
    begin                : Thu Dec 13 2001
    copyright            : (C) 2001 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Monitor.h"

Monitor::Monitor():QFrame(0,"TrayImonc-Monitor",WStyle_Customize | WStyle_DialogBorder)
{
  KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	OnTop = config->readBoolEntry("OnTop");
	OnAll = config->readBoolEntry("OnAllDesktops");
	frame = config->readBoolEntry("ShowFrame");
	
	mainFrame = new QFrame(this,"MainFrame");
	mainFrame->setMaximumSize(130,70);
	mainFrame->setMaximumSize(130,70);
	
	int posX = config->readNumEntry("posX",-1);
	int posY = config->readNumEntry("posY",-1);
	
	if(posX>0 && posY>0)
	{
	 	move(QPoint(posX,posY));
	}
	else
	{
		//Holen der Position des Kickers und dessen Gre
		//um den TrafficMonitor direkt neben dem Kicker zu positionieren
 		DCOPClient *client = new DCOPClient();
		client->attach();
	
		QByteArray data, replyData;
		QCString replyType;
		position = -1;
		kickerSize = -1;
	
		QDataStream arg(data, IO_WriteOnly);
		arg << 5;
		if (!client->call("kicker", "Panel", "panelPosition()",data, replyType, replyData))
			qDebug("there was some error using DCOP.");
		else
		{
  		QDataStream reply(replyData, IO_ReadOnly);
  		if (replyType == "int")
  		{
    		reply >> position;
    		if(position == 0 || position == 2)
    		{
    			if (!client->call("kicker", "Panel", "panelSize()",data, replyType, replyData))
    				qDebug("there was some error using DCOP.");
					else
					{
  					QDataStream reply(replyData, IO_ReadOnly);
  					if (replyType == "int")
  					{
    					reply >> kickerSize;
    				
    					switch(kickerSize)
    					{
    				 		case 0: kickerSize = 24;break;
    				 		case 1: kickerSize = 30;break;
    				 		case 3: kickerSize = 58;break;
    				 		default: kickerSize = 46;
    					}
    				}	
    			}
    		}
  		}
 		}

 		QRect r(QApplication::desktop()->geometry());
 	
 		switch(position)
 		{
 	 		case 0: move(QPoint(kickerSize,r.bottom()));break;
 	 		case 2: move(QPoint(r.right(),kickerSize));break;
 	 		default: move(QPoint(r.right(),r.bottom()));
 		}
 	}
 	
 	setCaption("Monitor");
 	
 	QGridLayout *grid = new QGridLayout(this,1,1);
 	oszi = new Oszi(mainFrame);
 	
 	vbox = new QVBoxLayout(mainFrame);
 	vbox->addWidget(oszi);
 	hbox = new QHBox(mainFrame);
 	dRate = new Rate(hbox,red,"D: ","KB/s");
 	uRate = new Rate(hbox,yellow,"U: ","KB/s");
 	vbox->addWidget(hbox);
 	grid->addWidget(mainFrame,0,0);
 	
 	menu = new QPopupMenu(this);
 	
 	menu->insertItem(i18n("Stay On Top"),this,SLOT(onTop()),0,0,-1);
 	menu->setItemChecked(0,OnTop);
 	menu->insertItem(i18n("On all Desktops"),this,SLOT(OnAllDesktops()),0,4,-1);
	menu->setItemChecked(4,OnAll);
	menu->insertItem(i18n("Show Frame"),this,SLOT(showFrame()),0,6,-1);
	menu->setItemChecked(6,frame);
 	menu->insertItem(i18n("Hangup"),this,SIGNAL(hangup()),0,2,-1);
 	menu->insertSeparator(3);
 	menu->insertItem(i18n("Save Position"),this,SLOT(savePosition()),0,5,-1);
 	menu->insertItem(SmallIcon("configure"),i18n("Options"),this,SIGNAL(options()),0,3,-1);
 	menu->insertSeparator(6);
 	menu->insertItem(SmallIcon("fileclose"),i18n("Close"),this,SLOT(closeMonitor()),0,1,-1);
 	
 	connect(oszi,SIGNAL(rightclicked(QPoint)),SLOT(rightclicked(QPoint)));
 	connect(oszi,SIGNAL(wantToMove(int,int)),SLOT(wantToMove(int,int)));
 	connect(oszi,SIGNAL(moveReady()),SLOT(moveReady()));
 	connect(dRate,SIGNAL(rightclicked(QPoint)),SLOT(rightclicked(QPoint)));
 	connect(dRate,SIGNAL(wantToMove(int,int)),SLOT(wantToMove(int,int)));
 	connect(dRate,SIGNAL(moveReady()),SLOT(moveReady()));
 	connect(uRate,SIGNAL(rightclicked(QPoint)),SLOT(rightclicked(QPoint)));
 	connect(uRate,SIGNAL(wantToMove(int,int)),SLOT(wantToMove(int,int)));
 	connect(uRate,SIGNAL(moveReady()),SLOT(moveReady()));
	
	delete(config);
}

Monitor::~Monitor()
{
}

void Monitor::show()
{	
 	//Eigentlich sollten die folgenden Schritte im Konstruktor
 	//durchgefhrt werden. Es gibt jedoch Schwierigkeiten die Sichbarkeit
 	//auf allen Desktops wieder aufzuheben, wenn sie gleich im Konstruktor
 	//aktiviert wird
 	//Keine Ahnung warum, das mit dem KWin-Zeugs ist also noch nicht so zuverlssig
 	if(OnTop)
		KWin::setState(winId()	,NET::StaysOnTop | NET::SkipTaskbar);
	else
		KWin::setState(winId(),NET::SkipTaskbar);
	KWin::setOnAllDesktops(winId(),OnAll);
	
	if(frame)
	{
		clearMask();
	 	setMaximumSize(130,70);
		setMinimumSize(130,70);
	}
	else
	{
	 	setFrameStyle(QFrame::Box | QFrame::Raised);
		setLineWidth(3);
		setMask( QRegion(0,0,140,80, QRegion::Rectangle) );
		setMaximumSize(140,80);
		setMinimumSize(140,80);	
	}
	
	QFrame::show();
}

void Monitor::closeEvent(QCloseEvent* event)
{
	emit closeTheMonitor();	
}

void Monitor::onTop()
{
	//Das Fenster muss erst versteckt und dann wieder angezeigt werden,
	//damit die nderungen auf allen Desktops auch bernommen werden
	hide();
	KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	
	if(OnTop)
	{
		OnTop = false;
	}
	else
	{
		OnTop = true;
	}
	
	show();
 	menu->setItemChecked(0,OnTop);
	config->writeEntry("OnTop",OnTop);
	config->sync();	
	
	delete(config);		
}

void Monitor::OnAllDesktops()
{
	//Das Fenster muss erst versteckt und dann wieder angezeigt werden,
	//damit die nderungen auf allen Desktops auch bernommen werden
	hide();
	KSimpleConfig *config = new KSimpleConfig("TrayImonc");

	if(OnAll)
		OnAll = false;
	else
		OnAll = true;
	
	show();
	menu->setItemChecked(4,OnAll);
	config->writeEntry("OnAllDesktops",OnAll);
	config->sync();
	
	delete(config);
}

void Monitor::savePosition()
{
	KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	QRect frame = geometry();
	config->writeEntry("posX",frame.left());
	config->writeEntry("posY",frame.top());
	config->sync();

	delete(config);	
}

void Monitor::closeMonitor()
{
	emit closeTheMonitor();	
}

void Monitor::setRate(int in, int out)
{
	double input = (double)in/1024;
	dRate->setRate(QString::number(input,'f',1));
	double output = (double)out/1024;
	uRate->setRate(QString::number(output,'f',1));
	oszi->paintRate(in/1024,out/1024);
}

void Monitor::rightclicked(QPoint point)
{
	menu->exec(point);	
}

void Monitor::changeColors(QColor inColor,QColor outColor)
{
 	this->inColor = inColor;
 	this->outColor = outColor;
 	oszi->setColors(inColor,outColor);
 	dRate->setColor(inColor);
 	uRate->setColor(outColor);
}

void Monitor::showFrame()
{
	KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	QRect frameRect = geometry();
	int posX = frameRect.left();
	int posY = frameRect.top();
	
	hide();
	
	if(frame)
	 	frame = false;
	else
	 	frame = true;
	
	show();
	
	move(QPoint(posX,posY));
	
	menu->setItemChecked(6,frame);
	config->writeEntry("ShowFrame",frame);
	config->sync();

	delete(config);
}

void Monitor::moveWidget(int X,int Y)
{	
 	move(QPoint(X-diffX,Y-diffY));
}

void Monitor::wantToMove(int X,int Y)
{
	QRect frameRect = frameGeometry();
	int posX = frameRect.left();
	int posY = frameRect.top();
	diffX = X-posX;
	diffY = Y-posY;
	connect(oszi,SIGNAL(moveWidget(int,int)),SLOT(moveWidget(int,int)));
 	connect(dRate,SIGNAL(moveWidget(int,int)),SLOT(moveWidget(int,int)));
 	connect(uRate,SIGNAL(moveWidget(int,int)),SLOT(moveWidget(int,int)));
}

void Monitor::moveReady()
{
	disconnect(oszi,SIGNAL(moveWidget(int,int)),0,0);
	disconnect(dRate,SIGNAL(moveWidget(int,int)),0,0);
	disconnect(uRate,SIGNAL(moveWidget(int,int)),0,0);
}
