/***************************************************************************
                          OptionsDialog.cpp  -  description
                             -------------------
    begin                : Fri Jan 25 2002
    copyright            : (C) 2002 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "OptionsDialog.h"

// savelog
#include <qdir.h>
// <--

OptionsDialog::OptionsDialog(bool online):QDialog(0,i18n("Options"),true)
{
	setCaption(i18n("TrayImonc -- Options"));
			
	allg = new QFrame(this);
	traffic = new QFrame(this);
	status = new QFrame(this);
	calls = new QFrame(this);
	menu = new QFrame(this);
	buttonFrame = new QFrame(this);

	stack = new QWidgetStack(this);
	stack->addWidget(allg,0);
	stack->addWidget(traffic,1);
	stack->addWidget(menu,2);
	
	tabBar = new QTabBar(this);
	tabBar->resize(300,50);
	allgTab = new QTab(i18n("General"));
	tabBar->addTab(allgTab);
	trafficTab = new QTab(i18n("TrafficMonitor"));

	if(!online)
		trafficTab->setEnabled(false);

	tabBar->addTab(trafficTab);
	menuTab = new QTab(i18n("Menu settings"));
	tabBar->addTab(menuTab);
	
	grid = new QGridLayout(this,5,1);
	grid->addWidget(tabBar,0,0);
	grid->addWidget(stack,1,0);
	grid->addWidget(buttonFrame,2,0,Qt::AlignCenter);
	

// general options tab
	
	allggrid = new QGridLayout(allg,6,2,8);
	serverlabel = new QLabel(allg);
	serverlabel->setText(i18n("Server name:"));
	serveredit = new QLineEdit(allg);
	portlabel = new QLabel(allg);
	portlabel->setText(i18n("Port:"));
	portedit = new QLineEdit(allg);
	telportlabel = new QLabel(allg);
	telportlabel->setText(i18n("Telmond port:"));
	telportedit = new QLineEdit(allg);
	passlabel = new QLabel(allg);
	passlabel->setText(i18n("Password:"));
	passedit = new QLineEdit(allg);
	passedit->setEchoMode(QLineEdit::Password);
	
	fontbutton = new QPushButton(i18n("Call Font"),allg);
	fontlabel = new QLabel(allg);
	connect(fontbutton,SIGNAL(clicked()),this,SLOT(fontSelected()));
	
	calltimelabel = new QLabel(i18n("Call notification time (ms):"),allg);
	calltimespinbox = new QSpinBox(0,65535,1,allg);
	callbuttonBox = new QCheckBox(i18n("Call notification close button"), allg);
	
	statusBox = new QCheckBox(i18n("Show state window on startup"), allg);
	connect(tabBar,SIGNAL(selected(int)),SLOT(tabSelected(int)));
	
	// savelog
	logBox = new QCheckBox(i18n("Save logs"), allg);
	logLabel = new QLabel(allg);
	logLabel->setText(i18n("Log file:"));
	logEdit = new QLineEdit(allg);
	// <--
	
	allggrid->addWidget(serverlabel, 0, 0 );
	allggrid->addWidget(serveredit, 0, 1);
	allggrid->addWidget(portlabel, 1, 0 );
	allggrid->addWidget(portedit, 1, 1);
	allggrid->addWidget(telportlabel, 2, 0);
	allggrid->addWidget(telportedit, 2, 1);
	allggrid->addWidget(passlabel, 3, 0 );
	allggrid->addWidget(passedit, 3, 1);
	allggrid->addWidget(fontlabel, 4, 0);
	allggrid->addWidget(fontbutton, 4, 1);
	allggrid->addWidget(calltimelabel,5,0);
	allggrid->addWidget(calltimespinbox,5,1);
	allggrid->addWidget(callbuttonBox,6,0);
	allggrid->addWidget(statusBox,7,0);
	
	// savelog
	allggrid->addWidget(logBox,8,0);
	allggrid->addWidget(logLabel,9,0);
	allggrid->addWidget(logEdit,9,1);
	// <--
	
	allggrid->addColSpacing(0,100);
	allggrid->addColSpacing(1,180);
	allggrid->addRowSpacing(0,35);
	allggrid->addRowSpacing(1,35);
	allggrid->addRowSpacing(2,35);
// traffic monitor options tab
	
	trafficgrid = new QGridLayout(traffic,5,1,10);
	showtraffic = new QCheckBox(traffic);
	showtraffic->setText(i18n("Show TrafficMonitor"));
	
	colorFrame = new QFrame(traffic);
	colorLayout = new QGridLayout(colorFrame,2,3,0,5);
	inColorButton = new QPushButton(colorFrame);
	inColorButton->setMaximumSize(30,20);
	connect(inColorButton,SIGNAL(clicked()),this,SLOT(chooseInColor()));
	
	outColorButton = new QPushButton(colorFrame);
	outColorButton->setMaximumSize(30,20);
	connect(outColorButton,SIGNAL(clicked()),this,SLOT(chooseOutColor()));
	
	inColorLabel = new QLabel(colorFrame);
	inColorLabel->setText(i18n("Color Download-Rate"));
	outColorLabel = new QLabel(colorFrame);
	outColorLabel->setText(i18n("Color Upload-Rate"));
	colorLayout->addWidget(inColorButton,0,1);
	colorLayout->addWidget(inColorLabel,0,2);
	colorLayout->addWidget(outColorButton,1,1);
	colorLayout->addWidget(outColorLabel,1,2);
	
	colorLayout->setColStretch(0,10);
	colorLayout->setColStretch(1,2);
	colorLayout->setColStretch(2,100);
	
	ontop = new QCheckBox(traffic);
	ontop->setText(i18n("Always on top"));
	
	allDesktops = new QCheckBox(traffic);
	allDesktops->setText(i18n("Show on all virt. desktops"));
	
	showFrame = new QCheckBox(traffic);
	showFrame->setText(i18n("Show frame"));
	
	trafficgrid->addWidget(showtraffic,0,1);
	trafficgrid->addWidget(colorFrame,1,1);
	trafficgrid->addWidget(ontop,2,1);
	trafficgrid->addWidget(allDesktops,3,1);
	trafficgrid->addWidget(showFrame,4,1);
	
	buttons = new QHBox(buttonFrame);
	buttonGrid = new QGridLayout(buttonFrame,1,1);
	ok = new QPushButton(i18n("OK"),buttons);
	connect(ok,SIGNAL(clicked()),this,SLOT(OptionsOK()));
	cancel = new QPushButton(i18n("Cancel"),buttons);
	connect(cancel,SIGNAL(clicked()),this,SLOT(OptionsCancel()));
	buttonGrid->addWidget(buttons,0,0,Qt::AlignCenter);
	
// menu customizing tab
	menuLabel = new QLabel(i18n("Select visible menu entries:"),menu);
	menugrid = new QGridLayout(menu,15,3,1);
	statebox = new QCheckBox(i18n("State"),menu);
	routebox = new QCheckBox(i18n("Defaultroute"),menu);
	dialbox = new QCheckBox(i18n("Dial/Hangup"),menu);
	addlinkbox = new QCheckBox(i18n("Add link"),menu);
	rebootbox = new QCheckBox(i18n("Reboot"),menu);
	autobox = new QCheckBox(i18n("Auto"),menu);
	manualbox = new QCheckBox(i18n("Manual"),menu);
	offbox = new QCheckBox(i18n("Off"),menu);
	checkmailbox = new QCheckBox(i18n("Check mail"),menu);
	showcallsbox = new QCheckBox(i18n("Show incoming calls"),menu);	trafficmonitorbox = new QCheckBox(i18n("Show traffic monitor"),menu);
	optionsbox = new QCheckBox(i18n("Options"),menu);
	helpbox = new QCheckBox(i18n("Help"),menu);
	exitbox = new QCheckBox(i18n("Exit"),menu);
	
	menugrid->addWidget(menuLabel,0,1);
	menugrid->addWidget(statebox,1,1);
	menugrid->addWidget(routebox,2,1);
	menugrid->addWidget(dialbox,3,1);
	menugrid->addWidget(addlinkbox,4,1);
	menugrid->addWidget(rebootbox,5,1);
	menugrid->addWidget(autobox,6,1);
	menugrid->addWidget(manualbox,7,1);
	menugrid->addWidget(offbox,8,1);
	menugrid->addWidget(checkmailbox,9,1);
	menugrid->addWidget(showcallsbox,10,1);
	menugrid->addWidget(trafficmonitorbox,11,1);
	menugrid->addWidget(optionsbox,12,1);
	menugrid->addWidget(helpbox,13,1);
	menugrid->addWidget(exitbox,14,1);
	
// get settings from config and set values

	KSimpleConfig *config = new KSimpleConfig("TrayImonc");

	inColor.setRgb(config->readUnsignedNumEntry("inColor",4294967040UL));
	outColor.setRgb(config->readUnsignedNumEntry("outColor",4294901760UL));
	QPixmap inPix((inColorButton->size()).width()-5,(inColorButton->size()).height()-5);
	inPix.fill(inColor);
	inColorButton->setPixmap(inPix);
	QPixmap outPix((outColorButton->size()).width()-5,(outColorButton->size()).height()-5);
	outPix.fill(outColor);
	outColorButton->setPixmap(outPix);
	
	callfont = config->readFontEntry("CallFont");
	fontname = callfont.family() + " " + QString::number( callfont.pointSize() ) + "pt";
	if ( callfont.bold() )
		fontname += " Bold";
	if ( callfont.italic() )
		fontname += " Italic";
        fontlabel->setText( fontname );
	
	serveredit->setText(config->readEntry("Server","fli4l"));
	portedit->setText(config->readEntry("Port","5000"));
	telportedit->setText(config->readEntry("TelPort","5001"));
	passedit->setText(config->readEntry("Pass"));
	
	// savelog
	logEdit->setText(config->readEntry("Log",QDir::homeDirPath()+"/connections.log"));
	logBox->setChecked(config->readBoolEntry("SaveLog", false));
	logEdit->setEnabled(logBox->isChecked());
	connect(logBox, SIGNAL(stateChanged(int)),this,SLOT(enableDisable(int)));
	// <--
	
	showtraffic->setChecked(config->readBoolEntry("Trafficinfo",true));
	ontop->setChecked(config->readBoolEntry("OnTop",true));
	allDesktops->setChecked(config->readBoolEntry("OnAllDesktops",true));
	showFrame->setChecked(config->readBoolEntry("ShowFrame",true));
	
	statusBox->setChecked(config->readBoolEntry("Startup_status",true));
	callbuttonBox->setChecked(config->readBoolEntry("CallButton",false));
	calltimespinbox->setValue(config->readNumEntry("CallNotificationTime",5000));
	
	statebox->setChecked(config->readBoolEntry("MenuState",true));
	routebox->setChecked(config->readBoolEntry("MenuDefaultRoute",true));
	dialbox->setChecked(config->readBoolEntry("MenuDial",true));
	addlinkbox->setChecked(config->readBoolEntry("MenuAddLink",true));
	rebootbox->setChecked(config->readBoolEntry("MenuReboot",true));
	autobox->setChecked(config->readBoolEntry("MenuAuto",true));
	manualbox->setChecked(config->readBoolEntry("MenuManual",true));
	offbox->setChecked(config->readBoolEntry("MenuOff",true));
	checkmailbox->setChecked(config->readBoolEntry("MenuCheckMail",true));
	showcallsbox->setChecked(config->readBoolEntry("MenuShowCalls",true));
	trafficmonitorbox->setChecked(config->readBoolEntry("MenuTrafficMonitor",true));
	optionsbox->setChecked(config->readBoolEntry("MenuOptions",true));
	helpbox->setChecked(config->readBoolEntry("MenuHelp",true));
	exitbox->setChecked(config->readBoolEntry("MenuExit",true));
}

OptionsDialog::~OptionsDialog()
{
/*	delete(serveredit);
	delete(portedit);
	delete(passedit);
	delete(showtraffic);
	delete(ontop);
	delete(allDesktops);
	delete(statusBox);
	delete(allggrid);
	delete(serverlabel);
	delete(portlabel);
	delete(passlabel);
	delete(trafficgrid);
	delete(inColorLabel);
	delete(outColorLabel);
	delete(colorLayout);
	delete(colorFrame);
	delete(buttonGrid);
	delete(statusgrid);
	delete(allg);
	delete(traffic);
	delete(status);
	delete(stack);
	delete(buttonFrame);
	delete(tabBar);
	delete(grid);
*/
}

// savelog
void OptionsDialog::enableDisable(int status)
{
	if (status == 0) logEdit->setDisabled(true);
	else if (status == 2) logEdit->setEnabled(true);
}
// <--

void OptionsDialog::OptionsOK()
{
	KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	
	// save options to config file
	
	config->writeEntry("Server",serveredit->text());
	config->writeEntry("Port",portedit->text());
	config->writeEntry("TelPort",telportedit->text());
	config->writeEntry("Pass",passedit->text());
	config->writeEntry("CallFont",callfont);
	config->writeEntry("inColor",inColor.rgb());
	config->writeEntry("outColor",outColor.rgb());
	config->writeEntry("Startup_status",statusBox->isChecked());
	config->writeEntry("CallButton",callbuttonBox->isChecked());
	config->writeEntry("CallNotificationTime",calltimespinbox->value());
	config->writeEntry("MenuState",statebox->isChecked());
	config->writeEntry("MenuDefaultRoute",routebox->isChecked());
	config->writeEntry("MenuDial",dialbox->isChecked());
	config->writeEntry("MenuAddLink",addlinkbox->isChecked());
	config->writeEntry("MenuReboot",rebootbox->isChecked());
	config->writeEntry("MenuAuto",autobox->isChecked());
	config->writeEntry("MenuManual",manualbox->isChecked());
	config->writeEntry("MenuOff",offbox->isChecked());
	config->writeEntry("MenuCheckMail",checkmailbox->isChecked());
	config->writeEntry("MenuShowCalls",showcallsbox->isChecked());
	config->writeEntry("MenuTrafficMonitor",trafficmonitorbox->isChecked());
	config->writeEntry("MenuOptions",optionsbox->isChecked());
	config->writeEntry("MenuHelp",helpbox->isChecked());
	config->writeEntry("MenuExit",exitbox->isChecked());

	// savelog
	if (logEdit->text().contains("~")) logEdit->setText(logEdit->text().replace("~", QDir::homeDirPath()));
	config->writeEntry("Log",logEdit->text());
	config->writeEntry("SaveLog", logBox->isChecked());
	// <--
	
	config->sync();
	
	emit changeColors(inColor,outColor);
	emit updateFont();
	
	if(showtraffic->isChecked() != config->readBoolEntry("Trafficinfo"))
		emit TrafficInfo();
	if(ontop->isChecked() != config->readBoolEntry("OnTop"))
		emit OnTop();
	if(allDesktops->isChecked() != config->readBoolEntry("OnAllDesktops"))
		emit OnAllDesktops();
	if(showFrame->isChecked() != config->readBoolEntry("ShowFrame"))
		emit ShowFrame();
		
	emit updateMenu();
	hide();
}

void OptionsDialog::OptionsCancel()
{
	//OptionsDialog gecancelt
	//keine nderungen in Config-Datei
	hide();
}

void OptionsDialog::chooseInColor()
{
	QColor tempColor = inColor;
	int result = KColorDialog::getColor(tempColor,this);
	if(result == KColorDialog::Accepted)
	{
		inColor = tempColor;
		QPixmap inPix((inColorButton->size()).width()-5,(inColorButton->size()).height()-5);
		inPix.fill(inColor);
		inColorButton->setPixmap(inPix);
	}
}

void OptionsDialog::chooseOutColor()
{
	QColor tempColor = outColor;
	int result = KColorDialog::getColor(tempColor,this);
	if(result == KColorDialog::Accepted)
	{
		outColor = tempColor;
		QPixmap outPix((outColorButton->size()).width()-5,(outColorButton->size()).height()-5);
		outPix.fill(outColor);
		outColorButton->setPixmap(outPix);
	}
}

void OptionsDialog::tabSelected(int tab)
{
	stack->raiseWidget(tab);	
}

void OptionsDialog::fontSelected()
{
	bool ok;
	callfont = QFontDialog::getFont( &ok, callfont,this);
	if (ok)
	{
		fontname = callfont.family() + " " + QString::number( callfont.pointSize() ) + "pt";
		if ( callfont.bold() )
			fontname += " Bold";
		if ( callfont.italic() )
			fontname += " Italic";
		fontlabel->setText( fontname );
	}
}


