/***************************************************************************
                          Oszi.cpp  -  description
                             -------------------
    begin                : Sun Dec 16 2001
    copyright            : (C) 2001 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Oszi.h"

Oszi::Oszi(QWidget* parent):QWidget(parent,"Oszi")
{
	//Die Gre wird fest gesetzt und kann nicht verndert werden
	resize(130,50);
	setMaximumSize(130,50);
 	setMinimumSize(130,50);
 	
 	p = new QPainter(this);
 	p->setFont(QFont("helvetica",7));
 	
 	//Queues fr Up- und Downloadrate
 	in = new Queue(65);
 	out = new Queue(65);
}

Oszi::~Oszi()
{
	delete(p);
	delete(in);
	delete(out);
}

void Oszi::paintEvent(QPaintEvent *event)
{
	QWidget::paintEvent(event);
	
	//Hintergrund zeichnen
	p->fillRect(0,0,130,49,darkGray);
	p->setPen(QPen(lightGray,1));
	for(int i=9;i<=50;i +=10)
		p->drawLine(0,i,130,i);
	
	//Beschriftung der y-Achse	
	p->setPen(QPen(black,1));
	p->drawText(0,28,"50");
	
	//Abfragen des grten Wertes aus den beiden Queues,
	//um spter den Zoom-Faktor berechnen zu knnen	
	int prev = in->elementAt(0);
	int largest = in->largest();
	if(out->largest() > largest)
		largest = out->largest();
		
	//Ausrechnen des Zoomfaktors
	//es stehen 50 Punkte zur Verfgung, bei jedem 10er wird ein neuer
	//Zoomfaktor berechnet
	double zoom = (largest/10+1)*10;
	zoom = (double)50/zoom;
  QString str;
  QTextStream ts( &str, IO_WriteOnly );
  //ts << "Zoom: x" << form("%2.2F",zoom*2);
  ts.precision(2);
  ts << "Zoom: x" << zoom*2;
  p->drawText(0,7,str);
	
	//Zeichnen der Kurve fr die Downloadrate
	//jeder Wert wird hat eine horizontale Ausdehnung von 2 Punkten	
  p->setPen(QPen(inColor,1));
	for(int i=0;i<130;i +=2)
	{
		int wert = in->elementAt(i/2);
			p->drawLine(i,(int)(49-prev/(1/zoom)),i+2,(int)(49-wert/(1/zoom)));
		prev = wert;
	}
	
	//Uploadrate
	p->setPen(QPen(outColor,1));
	prev = out->elementAt(0);
	for(int i=0;i<130;i +=2)
	{
		int wert = out->elementAt(i/2);
			p->drawLine(i,(int)(49-prev/(1/zoom)-1),i+2,(int)(49-wert/(1/zoom)-1));
		prev = wert;
	}
	
}

void Oszi::paintRate(int input,int output)
{
	//hier kommen die Raten an
	//Eintrag der Werte in die Queues, Neuzeichnen
	in->enqueue(input);
	out->enqueue(output);
	repaint();
}

void Oszi::mousePressEvent (QMouseEvent* e)
{
	if(e->button() == RightButton)
	{
		emit rightclicked(e->globalPos());
	}
	else if(e->button() == LeftButton)
	{
 		emit(wantToMove(e->globalX(),e->globalY()));
 		setCursor(KCursor::sizeAllCursor());
	}
}

void Oszi::mouseReleaseEvent(QMouseEvent* e)
{
	if(e->button() == LeftButton)
	{
 		emit(moveReady());
 		setCursor(KCursor::arrowCursor());
 	}
} 	

void Oszi::mouseMoveEvent(QMouseEvent* e)
{
	emit moveWidget(e->globalX(),e->globalY());
}

void Oszi::setColors(QColor inColor,QColor outColor)
{
	this->inColor = inColor;
	this->outColor = outColor;
	repaint();
}

Elem::Elem(int wert,Elem *next)
{
	this->wert = wert;
	this->next = next;	
}

Elem::~Elem()
{
	
}

Queue::Queue(int size)
{
	theSize = size;
	Elem *theLast = NULL;
	Elem *elem;
	
	//Alle Elemente der neuen Queue werden mit 0 initialisiert
	for(int i=0;i<size;i++)
	{
	 	elem = new Elem(0,theLast);
	 	if(theLast == NULL)
	 		last = elem;
	 	theLast = elem;
	}
	first = elem;
	theLargest = 0;
}

Queue::~Queue()
{
	//Freigabe aller Elemente der Queue
	Elem *elem = first;
	for(int i=0;i<theSize;i++)
	{
		if(first != last)
		{
			elem = elem->next;
			delete(first);
			first = elem;
		}
		else
			delete first;
	}
}

void Queue::enqueue(int wert)
{
	//Einfgen eines neuen Wertes in die Queue
	//der Wert des aktuell ersten Elementes der Queue wird neu gesetzt
	//danach wird dieses Element am Ende der Queue positioniert
	//alle anderen Elemente rutschen dadurch um eine Position nach vorn
	int temp = first->wert;
	Elem *elem = first;
	elem->wert = wert;
	first = elem->next;
	last->next = elem;
	elem->next = NULL;
	last = elem;
	
	//Neue Festlegung des grten Wertes, falls der eingefgte Wert
	//grer als das bisherige grte Element ist
	if(temp == theLargest || wert >= theLargest)
		findLargest();
}

int Queue::elementAt(int pos)
{
	//Rckgabe des Wertes an einer bestimmten Position der Queue
	Elem *elem = first;
	if(pos <= theSize)
		for(int i=0;i<pos;i++)
			elem = elem->next;
	else return -1;
	
 	return elem->wert;
}

int Queue::size()
{
 	return theSize;
}

int Queue::largest()
{
	return theLargest;
}

void Queue::findLargest()
{
	Elem *elem = first;
	theLargest = first->wert;
	while(elem->next != NULL)
	{
		elem = elem->next;
		if(elem->wert > theLargest)
			theLargest = elem->wert;	 	
	}
}

