/***************************************************************************
                          Router.cpp  -  description
                             -------------------
    begin                : Fri Oct 19 2001
    copyright            : (C) 2001 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Router.h"

Router::Router()
{
	talk = NULL;
	ISDN = false;
	DSL = false;
	DSLonline = false;
	DSLtraffic = false;
	ISDNone = false;
	ISDNtwo = false;
	ISDNonetraffic = false;
	ISDNtwotraffic = false;
	mReboot = false;
	mHalt = false;
	mReset = false;
	reConnect = false;
	route = -1;
	aktIn = 0;
	aktOut = 0;
	dialMode = "hossa";
	config = new KSimpleConfig("TrayImonc");
}

Router::~Router()
{
	delete(talk);
}

void Router::connectToRouter()
{
	if(talk == NULL)
	{
	 	talk = new Fli4lTalk();
	 	
	 	connect(this,SIGNAL(connectToHost(QString,int)),talk,SLOT(connectToHost(QString,int)));
	 	connect(talk,SIGNAL(connected()),SLOT(connectedSlot()));
	 	connect(talk,SIGNAL(lostConnection()),SLOT(disconnectedSlot()));
	 	connect(talk,SIGNAL(connectFailed()),SLOT(errorSlot()));
	 	
		QString host = config->readEntry("Server");
		uint port = config->readUnsignedNumEntry("Port");
		if(host == NULL || host == "")
			host = "fli4l";
		if(port == 0)
			port = 5000;
		emit connectToHost(host,port);
	}
}

void Router::connectedSlot()
{
	QString answer;
	QString password;
	int status = 0;
	
	//do we have to use a password?
	talk->sendCommand("pass");
	answer = talk->getAnswer();
	if(answer.startsWith("1"))
	{
		password = config->readEntry("Pass");
		if(password == NULL || password == "")
			password = QInputDialog::getText("TrayImonc","Please enter the fli4l password",QLineEdit::Password);
		QString str;
		QTextStream ts( &str, IO_WriteOnly );
		ts << "pass " << password;
		talk->sendCommand(str);
		answer = talk->getAnswer();
		if(!answer.startsWith("5"))
			return;
	}
	
	//Test ob DSL verwendet wird
	talk->sendCommand("status pppoe");
	answer = talk->getAnswer();
	if(!answer.startsWith("ERR"))
	{
		status++;
		DSL = true;
	}
	
	//Test ob ISDN vorhanden ist
	talk->sendCommand("status 1");
	answer = talk->getAnswer();
	if(!answer.startsWith("ERR"))
	{
		status +=2;
		ISDN = true;
	}
	else
		emit noISDN();
		
	controlTimer = new QTimer(this);
	connect(controlTimer,SIGNAL(timeout()),this,SLOT(controlSlot()));
 	controlTimer->start(1000, true);
 	
 	talk->sendCommand("circuits");
	int circuit_count = (talk->getAnswer()).toInt();
 	
 	for(int i=0;i<circuit_count;i++)
 	{
 		QString str;
    QTextStream ts( &str, IO_WriteOnly );
    ts << "circuit " << i+1;
 		talk->sendCommand(str);
 		answer = talk->getAnswer();
 		emit addCircuit(answer);
 	}
 	
	emit connectedSig(status,password);
}

void Router::disconnectedSlot()
{
	if (!reConnect)
	{
		reConnect = true;
		QString host = config->readEntry("Server");
		uint port = config->readUnsignedNumEntry("Port");
		if(host == NULL || host == "")
			host = "fli4l";
		if(port == 0)
			port = 5000;
		emit connectToHost(host,port);
	}
	else
	{
		reConnect = false;
		delete(talk);
		talk = NULL;
		emit disconnectedSig();
	}
}

void Router::errorSlot()
{
	delete(talk);
	talk = NULL;  	
	emit couldnotconnect();
}

void Router::controlSlot()
{
	QString answer;
  int iconState = 0;
  aktIn = 0;
  aktOut = 0;
	
	//Abfrage der Default-Route
	talk->sendCommand("route");
	answer = talk->getAnswer();
	if(answer == NULL)
    return;
	
  int defroute = answer.toInt();
	if(defroute != route)
	{
		emit setDefaultCircuit(defroute);		
		route = defroute;
	}
	
	//Abfrage des Whlmodus
	talk->sendCommand("dialmode");
	answer = talk->getAnswer();
	if(answer == NULL)
    return;

	if(answer == "auto" && dialMode != "auto")
	{
		emit autodial();
		dialMode = "auto";
	}
	else if(answer == "manual" && dialMode != "manual")
	{
		emit manualdial();
		dialMode = "manual";
	}
	else if(answer == "off" && dialMode != "off")
	{
		emit offdial();
		dialMode = "off";
	}
	
	//Abfrage des DSL-Status insofern vorhanden
	if(DSL)
	{
		talk->sendCommand("status pppoe");
		answer = talk->getAnswer();
		if(answer == NULL)
      return;
		
		if(answer == "Offline")
		{
      if(DSLonline)
      {
			  DSLonline = false;
				DSLtraffic = false;
				emit dsloffline();
        emit setToolTip("Offline");
      }
      iconState += 100;
		}
		else if(answer == "Online")
		{
			talk->sendCommand("rate pppoe");
			answer = talk->getAnswer();
				
			if(!DSLonline)
			{
				DSLonline = true;
        iconState += 200;
				emit dialed();
        QString tooltip;
        QTextStream tt( &tooltip, IO_WriteOnly );
        talk->sendCommand("phone pppoe");
        tt << "Online " << talk->getAnswer();
        emit setToolTip(tooltip);
			}
			if(parseRate(answer))
			{
				if(!DSLtraffic)
				{
					DSLtraffic = true;
				}
        iconState += 300;
			}
			else
			{	
				if(DSLtraffic)
				{
					DSLtraffic = false;
				}
         iconState += 200;
			}
		}
	}

	
	//Abfrage ISDN-Status insofern vorhanden
	if(ISDN)
	{
		talk->sendCommand("status 1");
		answer = talk->getAnswer();
		if(answer == NULL)
      return;
	
		if(answer == "Offline")
		{
      if(ISDNone)
      {
      	ISDNone = false;
      	ISDNonetraffic = false;
			  emit isdnoffline();
        emit setToolTip("Offline");
      }
      iconState += 10;
		}
		else if(answer == "Online")
		{
			talk->sendCommand("rate 1");
			answer = talk->getAnswer();
				
      if(!ISDNone)
			{
				ISDNone = true;
        iconState += 20;
				emit isdndialed();
        QString tooltip;
        QTextStream tt( &tooltip, IO_WriteOnly );
        talk->sendCommand("phone 1");
        tt << "Online " << talk->getAnswer();
        emit setToolTip(tooltip);
			}
				
			else if(parseRate(answer))
			{
				if(!ISDNonetraffic)
				{
					ISDNonetraffic = true;
				}
        iconState += 30;
			}
			else
			{
				if(ISDNonetraffic)
				{
					ISDNonetraffic = false;
				}
        iconState += 20;
			}
		}

		talk->sendCommand("status 2");
		answer = talk->getAnswer();
		if(answer == NULL)
      return;

		if(answer == "Offline")
		{
      if(ISDNtwo)
      {
			  ISDNtwo = false;
			  ISDNtwotraffic = false;
			  emit isdntwooffline();
      }
      iconState += 1;
		}
		else if(answer == "Online")
		{
			talk->sendCommand("rate 2");
			answer = talk->getAnswer();
			
      if(!ISDNtwo)
			{
				ISDNtwo = true;
        iconState += 2;
				emit link_added();
			}
			else if(parseRate(answer))
			{
				if(!ISDNtwotraffic)
				{
					ISDNtwotraffic = true;
				}
        iconState += 3;
			}
			else
			{	
				if(ISDNtwotraffic)
  			{
					ISDNtwotraffic = false;
				}
        iconState += 2;
			}
		}
		
    emit changeIcon(iconState);
	}
	
	emit(setRate(aktIn,aktOut));
		
	//Reset der Telmond-Liste
	if(mReset)
	{
		talk->sendCommand("reset-telmond-log-file");
		talk->getAnswer();
	}

	//Kontrolle, ob ein Reboot gewnscht ist
	if(mReboot)
	{
		delete(controlTimer);
		controlTimer = NULL;
		talk->sendCommand("reboot");
		talk->getAnswer();
		delete(talk);
		talk = NULL;
		emit disconnectedSig();
	}
	
	//Kontrolle, ob Halt gewnscht ist
	else if(mHalt)
	{
		delete(controlTimer);
		controlTimer = NULL;
		talk->sendCommand("halt");
		talk->getAnswer();
		delete(talk);
		talk = NULL;
		emit disconnectedSig();
	}
	//neustarten des Timers
	//der Timer wird immer wieder neu gestartet und luft
	//also nicht stndig weiter, weil man nicht wei, wie lange
	//die Abfragen dauern und es sein knnte, dass die nchste
	//Abfrage schon wieder gestartet wird, whrend die vorherige noch luft
	else
		controlTimer->start(1000,true);
}

bool Router::parseRate(QString input)
{
	bool status = false;
	int start = input.find(" ");
	int output = (input.left(start)).toInt();
	aktIn += output;
	if(output > 0)
		status = true;
	
	output = (input.right((input.length()-start))).toInt();
	aktOut += output;
	if(output > 0)
		status = true;
	
	return status;
}

void Router::default_dial()
{
	if(DSLonline || ISDNone)
	{
		talk->sendCommand("hangup");
		talk->getAnswer();
	}
	else
	{
		talk->sendCommand("dial");
		talk->getAnswer();
	}
}

void Router::addLink()
{
	talk->sendCommand("route");
	QString route = talk->getAnswer();
	
	if(ISDNone && !ISDNtwo)
	{
		QString str;
  	QTextStream ts( &str, IO_WriteOnly );
  	ts << "addlink " << route;
		talk->sendCommand(str);
		talk->getAnswer();
	}
	else if(ISDNtwo)
	{
		QString str;
  	QTextStream ts( &str, IO_WriteOnly );
  	ts << "removelink " << route;
		talk->sendCommand(str);
		talk->getAnswer();
	}
}

void Router::setDefaultRoute(int num)
{
	QString str;
  QTextStream ts( &str, IO_WriteOnly );
  ts << "route " << num;
 	talk->sendCommand(str);
	talk->getAnswer();
 	emit setDefaultCircuit(num);
}

void Router::setDialMode(QString modus)
{
	QString str;
  QTextStream ts( &str, IO_WriteOnly );
  ts << "dialmode " << modus;
 	talk->sendCommand(str);
	talk->getAnswer();
}

void Router::reboot()
{
	mReboot = true;
}

void Router::halt()
{
	mHalt = true;
}

void Router::resetTelmond()
{
 	mReset = true;
}
