/***************************************************************************
                          State.cpp  -  description
                             -------------------
    begin                : Tue Jan 15 2002
    copyright            : (C) 2002 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "State.h"
#include <iostream.h>
#include <qdir.h>

State::State():QWidget(0,"StateWidget")
{
	talk = new Fli4lTalk();

	setCaption("TrayImonc");
	resize(640,360);
	grid = new QGridLayout(this,2,1);
	stack = new QWidgetStack(this);
	tabBar = new QTabBar(this);
	tabBar->resize(539,50);
	status = new QTab(i18n("State"));
	tabBar->addTab(status);
	callTab = new QTab(i18n("Calls"));
	tabBar->addTab(callTab);
	connTab = new QTab(i18n("Connections"));
	tabBar->addTab(connTab);
	statusTab = new StatusTab(this);
	stack->addWidget(statusTab,0);
	callTable = new CallTab(this);
	stack->addWidget(callTable,1);
	connTable = new ConnTab(this);
	stack->addWidget(connTable,2);
	
	grid->addWidget(tabBar,0,0);
	grid->addWidget(stack,1,0);
	
	
	connect(tabBar,SIGNAL(selected(int)),SLOT(tabSelected(int)));
	connect(callTable,SIGNAL(updatePhoneBook()),SLOT(getTelmondLog()));
	connect(callTable,SIGNAL(reset()),SIGNAL(reset()));
	connect(statusTab,SIGNAL(dialMode(QString)),SIGNAL(dialMode(QString)));
	connect(statusTab,SIGNAL(setDefaultCircuit(int)),SIGNAL(setDefaultCircuit(int)));
}

State::~State()
{
	delete(grid);
	delete(statusTab);
	delete(callTable);
	delete(connTable);
	delete(stack);
	delete(tabBar);
	delete(talk);
}

void State::doConnect(int status,QString passwd)
{
	KSimpleConfig	*config = new KSimpleConfig("TrayImonc");

	this->passwd = passwd;

	connect(this,SIGNAL(connectToHost(QString,int)),talk,SLOT(connectToHost(QString,int)));
	connect(talk, SIGNAL(connected()),SLOT(connectedSlot()));
	//connect(talk, SIGNAL(lostConnection()),SLOT(disconnectedSlot()));
	
	QString host = config->readEntry("Server");
	uint port = config->readUnsignedNumEntry("Port");
	if(host == NULL || host == "")
		host = "fli4l";
	if(port == 0)
		port = 5000;
	//talk->connectToHost(host,port);
	emit connectToHost(host,port);
	
	delete(config);
}

void State::connectedSlot()
{
	KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	QString answer;
	bool DSL = false;
	bool ISDN = false;
	
	//Test ob Passwort bentigt wird
	talk->sendCommand("pass");
	answer = talk->getAnswer();
	
	if(answer.startsWith("1"))
	{
		QString str;
		QTextStream ts( &str, IO_WriteOnly );
  	ts << "pass " << passwd;
		talk->sendCommand(str);
		answer = talk->getAnswer();
    if(answer == NULL)
      return;
		if(!answer.startsWith("5"))
			return;
	}
	
	//Test ob DSL verwendet wird
	talk->sendCommand("status pppoe");
	answer = talk->getAnswer();
  if(answer == NULL)
      return;
	if(answer.length()>0)
		DSL = true;
	
	//Test ob ISDN vorhanden ist
	talk->sendCommand("status 1");
	answer = talk->getAnswer();
  if(answer == NULL)
      return;
	if(answer.length()>0)
		ISDN = true;
	
	if(DSL && ISDN)
	{
	 	kanaele = new QString[3];
	 	channelCount = 3;
	 	kanaele[0]= "pppoe";
	  kanaele[1]= "1";
	  kanaele[2]= "2";
	}
	else if(DSL)
	{
	 	kanaele = new QString[1];
	 	channelCount = 1;
	 	kanaele[0]= "pppoe";
	}
	else
	{
	 	kanaele = new QString[2];
	 	channelCount = 2;
	 	kanaele[0]= "1";	
	 	kanaele[1]= "2";
	}

 	getTelmondLog();

	// savelog
	getImondLog();
	// <--
 	
 	theTimer = new QTimer(this);
	connect(theTimer,SIGNAL(timeout()),this,SLOT(controlSlot()));
 	theTimer->start(1000, true);
 	
 	delete(config);
}

void State::disconnectedSlot()
{

}

void State::getTelmondLog()
{
	KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	QStringList sList;
	QString temp;
	QStringList phoneBook = config->readListEntry("PhoneBook");
	sList = talk->getList("telmond-log-file");
	callTable->clear();
	for ( QStringList::Iterator it = sList.begin(); it != sList.end(); ++it )
	{
		temp = (*it).latin1();
		
		if(temp.contains("ERR") || temp.contains("OK"))
			break;
	 	
		temp = temp.right(temp.length()-1);
		QString date = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-date.length()-1);
		QString time = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-time.length()-1);
		QString source = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-source.length()-1);
		QString aim = temp.left(temp.find("\n"));
		
		QString year = date.left(date.find("/"));
		date = date.right(date.length()-year.length()-1);
		QString month = date.left(date.find("/"));
		date = date.right(date.length()-month.length()-1);
		QString day = date.left(date.find("/"));
		date = "";
		QTextStream ts( &date, IO_WriteOnly );
		ts << day << "." << month << "." << year;
		QStringList match = phoneBook.grep(source);
		QString caller = i18n("unknown");
		if (match.join("") != NULL)
		{
			caller = match.join("");
			caller = caller.right(caller.length()-caller.findRev(":")-1);
		}

		callTable->addRow(date,time,caller,source,aim);
		statusTab->setCall(date,time,caller,source,aim);
	}
	emit updatePhoneBook();
}

// savelog
void State::getImondLog()
{
	KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	QStringList sList;
	QString temp;
	sList = talk->getList("imond-log-file");
	connTable->clear();
	for ( QStringList::Iterator it = sList.begin(); it != sList.end(); ++it )
	{
		temp = (*it).latin1();
		
		if(temp.contains("ERR") || temp.contains("OK"))
			break;
	 	
		temp = temp.right(temp.length()-1);
		QString provider = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-21);
		QString dateIn = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-dateIn.length()-1);
		QString timeIn = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-timeIn.length()-1);
		QString dateOut = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-dateOut.length()-1);
		QString timeOut = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-timeOut.length()-1);
		QString seconds = temp.left(5);
		temp = temp.right(temp.length()-6);
		QString secondsEff = temp.left(5);
		temp = temp.right(temp.length()-6);
		QString costs = temp.left(6);
		temp = temp.right(temp.length()-7);
		QString x = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-x.length()-1);
		QString trafficIn = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-trafficIn.length()-1);
		QString y = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-y.length()-1);
		QString trafficOut = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-trafficOut.length()-1);
		QString device = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-device.length()-1);
		QString secondsUnit = temp.left(temp.find(" "));
		temp = temp.right(temp.length()-secondsUnit.length()-1);
		QString costsUnit = temp.left(temp.find("\n"));
		
		QString yearIn = dateIn.left(dateIn.find("/"));
		dateIn = dateIn.right(dateIn.length()-yearIn.length()-1);
		QString monthIn = dateIn.left(dateIn.find("/"));
		dateIn = dateIn.right(dateIn.length()-monthIn.length()-1);
		QString dayIn = dateIn.left(dateIn.find("/"));
		dateIn = "";
		QTextStream ts( &dateIn, IO_WriteOnly );
		ts << dayIn << "." << monthIn << "." << yearIn;		
		
		QString yearOut = dateOut.left(dateOut.find("/"));
		dateOut = dateOut.right(dateOut.length()-yearOut.length()-1);
		QString monthOut = dateOut.left(dateOut.find("/"));
		dateOut = dateOut.right(dateOut.length()-monthOut.length()-1);
		QString dayOut = dateOut.left(dateOut.find("/"));
		dateOut = "";
		QTextStream ts2( &dateOut, IO_WriteOnly );
		ts2 << dayOut << "." << monthOut << "." << yearOut;

		connTable->addRow(provider,dateIn,timeIn,dateOut,timeOut,seconds,secondsEff,costs,x,trafficIn,y,trafficOut,device,secondsUnit,costsUnit);
	}
	connTable->setConnections(sList);
	if (config->readBoolEntry("SaveLog",false))
	{
		QString fileStr = QString(config->readEntry("Log", QDir::homeDirPath() + "/connections.log"));
		connTable->writeLogFile(sList, fileStr);
	}
}
// <--

void State::showState()
{
	//Falls das Statusfenster gerade auf einem anderen Desktop angezeigt
	//wird, wird es durch hide() und dann show() auf den aktuellen Desktop
	//gezaubert
	hide();
 	show();

  //Es gab Probleme, wenn das Statusfenster zum 2. Mal geffnet wurde
  //dabei wurde es immer nur minimiert angezeigt
  //hier wird das Statusfenster als aktives Fenster gesetzt
  KWin::setActiveWindow(winId());
}

void State::addRow(QString date,QString time,QString caller, QString source,QString aim)
{
	callTable->addRow(date,time,caller,source,aim);
	statusTab->setCall(date,time,caller,source,aim);
}

void State::tabSelected(int tab)
{
	stack->raiseWidget(tab);	
}

void State::dialAuto()
{
 	statusTab->setDialMode(0);
}

void State::dialManual()
{
	statusTab->setDialMode(1);
}

void State::dialOff()
{
	statusTab->setDialMode(2);
}

void State::addCircuit(QString circuit)
{
 	statusTab->addCircuit(circuit);
}

void State::setDefaultRoute(int num)
{
	statusTab->setDefaultRoute(num);
}

void State::setLastCall(QString date,QString time,QString caller,QString source,QString aim)
{
	statusTab->setCall(date,time,caller,source,aim);
}

void State::controlSlot()
{
	QString answer;
	
 	//Abfrage des Status der Kanaele
	for(int i=0;i<channelCount;i++)
	{
		QString inout;
		QTextStream inoutts(&inout, IO_WriteOnly );
	 	inoutts << "inout " << kanaele[i];
	 	talk->sendCommand(inout);
	 	answer = talk->getAnswer();
    if(answer == NULL)
      return;
	 	statusTab->setDirection(i,answer);
	 	
		QString ip;
		QTextStream ipts(&ip, IO_WriteOnly );
	 	ipts << "ip " << kanaele[i];
	 	talk->sendCommand(ip);
	 	answer = talk->getAnswer();
    if(answer == NULL)
      return;
	 	ip = answer;
	 	if(ip == "")
	 		statusTab->setIP(i,"0.0.0.0");
	 	else
	 		statusTab->setIP(i,answer);
		
	 	QString phone;
		QTextStream phonets(&phone, IO_WriteOnly );
  	phonets << "phone " << kanaele[i];
  	talk->sendCommand(phone);
	 	answer = talk->getAnswer();
    if(answer == NULL)
      return;
	 	if(ip == "" && kanaele[i] == "pppoe")
	 		answer = "";
	 	statusTab->setCircuit(i,answer);
	 		 	
	 	QString rate;
		QTextStream ratets(&rate, IO_WriteOnly );
	 	ratets << "rate " << kanaele[i];
	 	talk->sendCommand(rate);
	 	answer = talk->getAnswer();
    if(answer == NULL)
      return;
	 	statusTab->setRate(i,answer); 	
	 	
	 	QString atime;
		QTextStream atimets(&atime, IO_WriteOnly );
	 	atimets << "online-time " << kanaele[i];
	 	talk->sendCommand(atime);
	 	answer = talk->getAnswer();
    if(answer == NULL)
      return;
	 	statusTab->setATime(i,answer);
	 	
	 	QString otime;
		QTextStream otimets(&otime, IO_WriteOnly );
	 	otimets << "time " << kanaele[i];
	 	talk->sendCommand(otime);
	 	answer = talk->getAnswer();
    if(answer == NULL)
      return;
	 	statusTab->setOTime(i,answer);
	 	
	 	QString charge;
		QTextStream chargets(&charge, IO_WriteOnly );
	 	chargets << "charge " << kanaele[i];
	 	talk->sendCommand(charge);
	 	answer = talk->getAnswer();
    if(answer == NULL)
      return;
	 	statusTab->setCharge(i,answer);
	}
	
	//Datum und Zeit vom Router
	talk->sendCommand("date");
	answer = talk->getAnswer();
  if(answer == NULL)
      return;
	
	QString wday = answer.left(answer.find(" "));
	answer = answer.right(answer.length()-wday.length()-1);
	QString strdate = answer.left(answer.find(" "));
	answer = answer.right(answer.length()-strdate.length()-1);
	QString time = answer.left(answer.find("\n"));
	
  strdate.replace(strdate.find("/"),1,".");
  strdate.replace(strdate.find("/"),1,".");
	
  //hier sollte der Wochentag nicht stndig berprft werden
  //es wrde ausreichen, dies bei der Initialisierung zu tun
  if(wday == "Mo")
  	wday = i18n("Monday");
  else if(wday == "Tu")
  	wday = i18n("Tuesday");
  else if(wday == "We")
  	wday = i18n("Wednesday");
  else if(wday == "Th")
  	wday = i18n("Thursday");
  else if(wday == "Fr")
  	wday = i18n("Friday");
  else if(wday == "Sa")
  	wday = i18n("Saturday");
 	else if(wday == "Su")
  	wday = i18n("Sunday");

  answer = "";
  QTextStream ts( &answer, IO_WriteOnly );
  ts << wday << " " << strdate << " " << time;
  statusTab->setDate(answer);

  //CPU-Auslastung
  talk->sendCommand("cpu");
  answer = talk->getAnswer();
  if(answer == NULL)
      return;

  QString out;
	QTextStream ts1( &out, IO_WriteOnly );
  ts1 << answer << " %";
  statusTab->setCPU(out);
	
	//Uptime
	talk->sendCommand("uptime");
	answer = talk->getAnswer();
  if(answer == NULL)
      return;
	int sec = answer.toInt();
	int days = sec/86400;
	sec = sec%86400;
	int hours = sec/3600;
	sec = sec%3600;
	int min = sec/60;
	sec = sec%60;
	out = "";
	QTextStream ts2( &out, IO_WriteOnly );
	if(days > 1)
  	ts2 << days << " " << i18n("Days") << " ";
 	else if(days > 0)
 		ts2 << days << " " << i18n("Day") << " ";
  if(hours < 10)
  	ts2 << "0";
  ts2 << hours << ":";
  if(min < 10)
  	ts2 << "0";
  ts2 << min << ":";
  if(sec < 10)
  	ts2 << "0";
  ts2 << sec;
  statusTab->setUptime(out);
	
	theTimer->start(1000,true);
}
