/***************************************************************************
                          StatusTab.cpp  -  description
                             -------------------
    begin                : Sun Jan 20 2002
    copyright            : (C) 2002 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "StatusTab.h"

StatusTab::StatusTab(QWidget *parent):QFrame(parent)
{
	channelState = new QString[3];
	channelState[0] = "nix";
	channelState[1] = "nix";
	channelState[2] = "nix";
	grid = new QGridLayout(this,3,3,5);
	dateTime = new QGroupBox(1,Qt::Vertical,i18n("Date / Time"),this);
	cpu = new QGroupBox(1,Qt::Vertical,i18n("CPU"),this);
	up = new QGroupBox(1,Qt::Vertical,i18n("Uptime"),this);
	channels = new QGroupBox(1,Qt::Vertical,i18n("Channels"),this);
	route = new QGroupBox(1,Qt::Vertical,i18n("Route"),this);
	mode = new QButtonGroup(3,Qt::Vertical,i18n("Dialmode"),this);
	
	grid->addWidget(dateTime,0,0);
	grid->addWidget(up,0,1);
	grid->addWidget(cpu,0,2);
	grid->addMultiCellWidget(channels,1,1,0,2);
	grid->addMultiCellWidget(route,2,2,0,1);
	grid->addWidget(mode,2,2);
	
	grid->setColStretch(0,100);
	grid->setColStretch(1,100);
	grid->setColStretch(2,50);
	
 	pix1 = KGlobal::iconLoader()->loadIcon("isdn_offl.png",KIcon::Small);
 	pix2 = KGlobal::iconLoader()->loadIcon("isdn_online.png",KIcon::Small);
 	
 	theDate = new QLabel(dateTime);
 	theCPU = new QLabel(cpu);
 	theUptime = new QLabel(up);
 	
	channelTable = new Table(3,8,channels);
	channelTable->setSelectionMode(QTable::NoSelection);
	channelTable->setLeftMargin(0);
	QFont f("Helvetica",8);
	channelTable->setFont(f);
	col = channelTable->horizontalHeader();
	channelTable->verticalHeader()->hide();
	col->setClickEnabled(false);
	col->setLabel(0,i18n("Route"));
 	col->resizeSection(0,80);
 	col->setLabel(1,i18n("Direction"));
 	col->resizeSection(1,55);
 	col->setLabel(2,i18n("IP"));
 	col->resizeSection(2,90);
 	col->setLabel(3,i18n("In"));
 	col->resizeSection(3,60);
 	col->setLabel(4,i18n("Out"));
 	col->resizeSection(4,60);
 	col->setLabel(5,i18n("OnlTime"));
 	col->resizeSection(5,50);
 	col->setLabel(6,i18n("Time"));
 	col->resizeSection(6,50);
 	col->setLabel(7,i18n("Cost"));
 	col->resizeSection(7,45);
 	
 	channelTable->setColumnStretchable(0,true);
 	
 	dialAuto = new QRadioButton(i18n("Auto"),mode);
 	manual = new QRadioButton(i18n("Manual"),mode);
 	off = new QRadioButton(i18n("Off"),mode);
 	
 	routeFrame = new QFrame(route);
 	routeGrid = new QGridLayout(routeFrame,2,2,10);
 	defLabel = new QLabel(i18n("Default Route:"),routeFrame);
 	routeGrid->addWidget(defLabel,0,0,Qt::AlignLeft);
 	box = new QComboBox(routeFrame);
 	box->insertItem(i18n("Automatic (LCR)"));
 	routeGrid->addWidget(box,0,1,Qt::AlignLeft);
 	routeGrid->setColStretch(0,25);
 	routeGrid->setColStretch(1,75);
 	callLabel = new QLabel(i18n("Last Call:"),routeFrame);
 	routeGrid->addWidget(callLabel,1,0,Qt::AlignLeft);
 	lastCall = new QLabel("--",routeFrame);
 	routeGrid->addWidget(lastCall,1,1,Qt::AlignLeft);
 	
 	connect(box,SIGNAL(activated(int)),SIGNAL(setDefaultCircuit(int)));
 	connect(mode,SIGNAL(clicked(int)),SLOT(changedDialMode(int)));
}

StatusTab::~StatusTab()
{		
	delete(channelTable);
	delete(channels);
	delete(theDate);
	delete(dateTime);
	delete(theCPU);
	delete(cpu);
	delete(theUptime);
	delete(up);
	delete(lastCall);
	delete(box);
	delete(dialAuto);
 	delete(manual);
	delete(off);
	delete(mode);
	delete(routeGrid);	
	delete(defLabel);
	delete(callLabel);
	delete(routeFrame);
 	delete(route);
	delete(grid);
}

void StatusTab::setDate(QString aktDate)
{
	theDate->setText(aktDate);			
}

void StatusTab::setCPU(QString aktCPU)
{
	theCPU->setText(aktCPU);
}

void StatusTab::setUptime(QString aktUptime)
{
	theUptime->setText(aktUptime);
}

void StatusTab::setCall(QString date,QString time,QString caller,QString source,QString aim)
{
		QString call;
		QTextStream labelText(&call, IO_WriteOnly );
		if (caller == i18n("unknown"))
			labelText << source << " --> " << aim << "(" << date << ", " << time << ")";
		else
			labelText << caller << " --> " << aim << "(" << date << ", " << time << ")";
 		lastCall->setText(call);
}

void StatusTab::setDialMode(int num)
{
	mode->setButton(num);	
}

void StatusTab::changedDialMode(int id)
{
	switch(id)
	{
	 	case 0: emit dialMode("auto");break;
	 	case 1: emit dialMode("manual");break;
	 	case 2: emit dialMode("off");break;
	}
}

void StatusTab::addCircuit(QString circuit)
{
	box->insertItem(circuit);
}

void StatusTab::setDefaultRoute(int num)
{
	box->setCurrentItem(num); 	
}

void StatusTab::setCircuit(int num,QString value)
{
	if(channelState[num] != value)
	{
		channelState[num] = value;
		if(value == "")
			channelTable->setPixmap(num,0,pix1);
		else
			channelTable->setPixmap(num,0,pix2);
		channelTable->setText(num,0,value);		
	}
}

void StatusTab::setDirection(int num,QString value)
{
	channelTable->setText(num,1,value);
}

void StatusTab::setIP(int num,QString value)
{
	if(value == "ERR")
		value ="0.0.0.0";
	channelTable->setText(num,2,value);
}

void StatusTab::setRate(int num,QString value)
{
	int in = (value.left(value.find(" "))).toInt();
	int out = (value.right(value.find(" "))).toInt();
	
	double fIn = (double)in/1024;
	double fOut = (double)out/1024;
	
	QString sIn;
	QTextStream ints( &sIn, IO_WriteOnly );
	ints << QString::number(fIn,'f',1) << " KB";
	QString sOut;
	QTextStream outts( &sOut, IO_WriteOnly );
	outts << QString::number(fOut,'f',1) << " KB";	
	channelTable->setText(num,3,sIn);
	channelTable->setText(num,4,sOut);
}

void StatusTab::setATime(int num,QString value)
{
	channelTable->setText(num,5,value);
}

void StatusTab::setOTime(int num,QString value)
{
	channelTable->setText(num,6,value);
}

void StatusTab::setCharge(int num,QString value)
{
	channelTable->setText(num,7,value);
}
