/***************************************************************************
                          Table.cpp  -  description
                             -------------------
    begin                : Wed Jan 16 2002
    copyright            : (C) 2002 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "Table.h"

Table::Table(int rows, int cols, QWidget *parent):QTable(rows,cols,parent)
{

}

Table::~Table()
{

}

//Im Folgenden werden Methoden von QTable berladen,
//um die grafische Darstellung der Header und Zellen der Tabelle
//nach meinen Wnschen anzupassen

//Konkret:
//- beim Selektieren einer Zeile, soll auch die angeklickte Zelle
//  farbig hinterlegt werden, so dass die ganze Zeile markiert ist
//- die aktive Zelle soll nicht umrahmt sein

void QTable::paintCell( QPainter* p, int row, int col,
			const QRect &cr, bool selected )
{
	int w = cr.width();
  int h = cr.height();
  int x2 = w - 1;
  int y2 = h - 1;

  QTableItem *itm = item( row, col );
  if ( itm )
  {
		p->save();
		itm->paint( p, colorGroup(), cr, selected );
		p->restore();
  }
  else
  {
		p->fillRect( 0, 0, w, h, selected ? colorGroup().brush( QColorGroup::Highlight ) : colorGroup().brush( QColorGroup::Base ) );
  }

  if ( sGrid )
  {
		// Draw our lines
		QPen pen( p->pen() );
		p->setPen( colorGroup().mid() );
		p->drawLine( x2, 0, x2, y2 );
		p->drawLine( 0, y2, x2, y2 );
		p->setPen( pen );
  }
}

void QTable::drawContents( QPainter *p, int cx, int cy, int cw, int ch )
{
		int colfirst = columnAt( cx );
    int collast = columnAt( cx + cw );
    int rowfirst = rowAt( cy );
    int rowlast = rowAt( cy + ch );

    if ( rowfirst == -1 || colfirst == -1 ) {
	paintEmptyArea( p, cx, cy, cw, ch );
	return;
    }

    if ( rowlast == -1 )
	rowlast = numRows() - 1;
    if ( collast == -1 )
	collast = numCols() - 1;

    // Go through the rows
    for ( int r = rowfirst; r <= rowlast; ++r ) {
	// get row position and height
	int rowp = rowPos( r );
	int rowh = rowHeight( r );

	if ( rowh == 0 )
	    continue;
	
	// Go through the columns in row r
	// if we know from where to where, go through [colfirst, collast],
	// else go through all of them
	for ( int c = colfirst; c <= collast; ++c ) {
	    // get position and width of column c
	    int colp, colw;
	    colp = columnPos( c );
	    colw = columnWidth( c );
	    int oldrp = rowp;
	    int oldrh = rowh;

	    if ( colw == 0 )
		continue;
	
	    QTableItem *itm = item( r, c );
	    if ( itm &&
		 ( itm->colSpan() > 1 || itm->rowSpan() > 1 ) ) {
		bool goon = r == itm->row() && c == itm->col() ||
			r == rowfirst && c == itm->col() ||
			r == itm->row() && c == colfirst;
		if ( !goon )
		    continue;
		rowp = rowPos( itm->row() );
		rowh = 0;
		int i;
		for ( i = 0; i < itm->rowSpan(); ++i )
		    rowh += rowHeight( i + itm->row() );
		colp = columnPos( itm->col() );
		colw = 0;
		for ( i = 0; i < itm->colSpan(); ++i )
		    colw += columnWidth( i + itm->col() );
	    }

	    // Translate painter and draw the cell
#ifdef QT_NO_TRANSFORMATIONS // ### Need both, or will the first do? (first is in qt/main)
	    p->translate( colp, rowp );
	    paintCell( p, r, c, QRect( colp, rowp, colw, rowh ), isSelected( r, c ) );
	    p->translate( -colp, -rowp );
#else
	    p->saveWorldMatrix();
	    p->translate( colp, rowp );
	    paintCell( p, r, c, QRect( colp, rowp, colw, rowh ), isSelected( r, c ) );
	    p->restoreWorldMatrix();
#endif

	    rowp = oldrp;
	    rowh = oldrh;
	}
    }

    // Paint empty rects
    paintEmptyArea( p, cx, cy, cw, ch );
}
