/***************************************************************************
                          TrayShell.cpp  -  description
                             -------------------
    begin                : Thu Oct 18 2001
    copyright            : (C) 2001 by Robert Nitschke
    email                : trayimonc@t-online.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "TrayShell.h"

TrayShell::TrayShell()
{
	//erzeugt das Icon, das im Tray angezeigt wird, wenn
	//keine Verbindung zum fli4l-Router besteht
        //aboutApp = new KAboutApplication(this);
        //optionDialog = new OptionsDialog();
	no_conn = new NoConn();
	
	showTraffic = false;
	online = false;
	
//	menu = NULL;
	
	connect(no_conn,SIGNAL(exitSig()),SLOT(exitApp()));
	connect(no_conn,SIGNAL(connectSig()),SLOT(connectApp()));
	connect(no_conn,SIGNAL(doubleclick()),SLOT(connectApp()));
	connect(no_conn,SIGNAL(options()),SLOT(options()));
	connect(no_conn,SIGNAL(about()),SLOT(about()));
	connect(no_conn,SIGNAL(help()),SLOT(help()));
	
	no_conn->show();
	
	//Versuchen Verbindung zum Router aufzubauen und
	//alle ntigen Instanzen zu erzeugen
	init();
}

TrayShell::~TrayShell()
{
  delete(aboutApp);
  //delete(optionDialog);
  delete(tray);
}

void TrayShell::show()
{
	//nix zum tun
}

void TrayShell::connectApp()
{
	init();
}

void TrayShell::init()
{
	KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	
	tray = new TrayIcon();
	calls = new Calls();
	monitor = new Monitor();
	state = new State();
	router = new Router();
	this->createMenu();

	connect(router, SIGNAL(connectedSig(int,QString)),SLOT(connectedSlot(int)));
	connect(router, SIGNAL(disconnectedSig()),SLOT(disconnectedSlot()));
	connect(router, SIGNAL(couldnotconnect()),SLOT(couldnotconnectSlot()));
	connect(router,SIGNAL(dialed()),SLOT(isOnline()));
	connect(router,SIGNAL(dsloffline()),SLOT(isOffline()));
	connect(router,SIGNAL(isdndialed()),SLOT(isOnline()));
	connect(router,SIGNAL(isdnoffline()),SLOT(isOffline()));
	connect(router, SIGNAL(changeIcon(int)),tray,SLOT(changeIcon(int)));
	connect(tray, SIGNAL(doubleclick()),router,SLOT(default_dial()));
	connect(router,SIGNAL(setToolTip(QString)),tray,SLOT(setToolTip(QString)));
	connect(router,SIGNAL(setRate(int,int)),monitor,SLOT(setRate(int,int)));
	connect(router,SIGNAL(disconnectedSig()),calls,SLOT(stop()));
	connect(monitor,SIGNAL(options()),SLOT(options()));
	connect(monitor,SIGNAL(hangup()),router,SLOT(default_dial()));
	connect(calls,SIGNAL(newCall(QString,QString,QString,QString,QString)),state,SLOT(addRow(QString,QString,QString,QString,QString)));

	// savelog
	connect(router,SIGNAL(isdnoffline()),state,SLOT(getImondLog()));
	connect(router,SIGNAL(dsloffline()),state,SLOT(getImondLog()));
	// <--

	connect(state,SIGNAL(reset()),router,SLOT(resetTelmond()));
	connect(state,SIGNAL(updatePhoneBook()),calls,SLOT(readPhoneBook()));
	connect(router,SIGNAL(autodial()),state,SLOT(dialAuto()));
	connect(router,SIGNAL(manualdial()),state,SLOT(dialManual()));
	connect(router,SIGNAL(offdial()),state,SLOT(dialOff()));
	connect(state,SIGNAL(dialMode(QString)),router,SLOT(setDialMode(QString)));
	connect(router,SIGNAL(addCircuit(QString)),state,SLOT(addCircuit(QString)));
	connect(state,SIGNAL(setDefaultCircuit(int)),router,SLOT(setDefaultRoute(int)));
	connect(router,SIGNAL(setDefaultCircuit(int)),state,SLOT(setDefaultRoute(int)));
	connect(router, SIGNAL(connectedSig(int,QString)),state,SLOT(doConnect(int,QString)));
	connect(this,SIGNAL(changeColors(QColor,QColor)),monitor,SLOT(changeColors(QColor,QColor)));
	
	unsigned int result;
	QColor inColor;
	QColor outColor;
	
	if((result = config->readUnsignedNumEntry("inColor"))==0)
		inColor = yellow;
	else
		inColor.setRgb(result);
	if((result = config->readUnsignedNumEntry("outColor"))==0)
		outColor = red;
	else
		outColor.setRgb(result);
	emit changeColors(inColor,outColor);
	
	delete(config);
	
	//Router-Instanz soll Verbindung zum Router aufbauen
	router->connectToRouter();
}

void TrayShell::exitApp()
{
	delete(monitor);
	delete(router);
	delete(tray);
	delete(menu);
	delete(calls);
	delete(no_conn);
	delete(state);
	emit quit();
}

void TrayShell::connectedSlot(int status)
{
	//Verbindung zum Router besteht, so dass
	//die Icons fr ISDN und/oder DSL im Tray angezeigt
	//werden knnen
	//No_conn-Objekt wird deshalb nicht mehr bentigt und freigegeben
	disconnect(no_conn,0,0,0);
	delete(no_conn);
	no_conn = NULL;

	tray->show();

	KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	
	if(config->readBoolEntry("Startup_status"))
		state->show();
		
	delete(config);
}

void TrayShell::disconnectedSlot()
{
	//Verbindung zum Router besteht nicht mehr
	//alle Objekte werden freigegeben und
	//ein neues no_conn-Objekt erzeugt und im Tray angezeigt
	state->hide();
	disconnect(router,0,0,0);
	disconnect(tray,0,0,0);
	disconnect(menu,0,0,0);
	disconnect(calls,0,0,0);
	disconnect(state,0,0,0);
	disconnect(monitor,0,0,0);
	delete(router);
	delete(tray);
	delete(menu);
	delete(calls);
	delete(state);
	delete(monitor);
	monitor = NULL;
	router = NULL;
	tray = NULL;
	menu = NULL;
	calls = NULL;
	state = NULL;
	
	no_conn = new NoConn();
	
	connect(no_conn,SIGNAL(exitSig()),SLOT(exitApp()));
	connect(no_conn,SIGNAL(connectSig()),SLOT(connectApp()));
	connect(no_conn,SIGNAL(options()),SLOT(options()));
	connect(no_conn,SIGNAL(about()),SLOT(about()));
	connect(no_conn,SIGNAL(help()),SLOT(help()));
//	connect(no_conn,SIGNAL(doubleclick()),SLOT(connectApp()));
	
	no_conn->show();
}

void TrayShell::couldnotconnectSlot()
{
	//Verbindung zum Router konnte nicht aufgebaut werden
	//keine nderungen im Status der Anwendung
	//no_conn wird weiterhin angezeigt
	cout << "could not connect" << endl;
	
	disconnect(router,0,0,0);
	disconnect(tray,0,0,0);
	disconnect(menu,0,0,0);
	disconnect(calls,0,0,0);
	disconnect(state,0,0,0);
	disconnect(monitor,0,0,0);
	delete(router);
	delete(tray);
	delete(menu);
	delete(calls);
	delete(state);
	delete(monitor);
	monitor = NULL;
	router = NULL;
	tray = NULL;
	menu = NULL;
	calls = NULL;
	state = NULL;
	
	if (!no_conn) 
	{
	  no_conn = new NoConn();
	  connect(no_conn,SIGNAL(exitSig()),SLOT(exitApp()));
	  connect(no_conn,SIGNAL(connectSig()),SLOT(connectApp()));
	  connect(no_conn,SIGNAL(options()),SLOT(options()));
	  connect(no_conn,SIGNAL(about()),SLOT(about()));
	  connect(no_conn,SIGNAL(help()),SLOT(help()));
//	connect(no_conn,SIGNAL(doubleclick()),SLOT(connectApp()));
	
	  no_conn->show();
	}
	
//	OptionsDialog optionDialog(false);
//	optionDialog.exec();
}

void TrayShell::about()
{
	//Anzeige des About-Dialoges
	//About-Data in main definiert
	KAboutApplication aboutApp(this);
  aboutApp.exec();
}

void TrayShell::options()
{
	if(menu != NULL)
	{
		OptionsDialog optionDialog(true);
		connect(&optionDialog,SIGNAL(TrafficInfo()),menu,SLOT(traffic_info()));
		connect(&optionDialog,SIGNAL(OnTop()),monitor,SLOT(onTop()));
		connect(&optionDialog,SIGNAL(OnAllDesktops()),monitor,SLOT(OnAllDesktops()));
		connect(&optionDialog,SIGNAL(changeColors(QColor,QColor)),monitor,SLOT(changeColors(QColor,QColor)));
		connect(&optionDialog,SIGNAL(ShowFrame()),monitor,SLOT(showFrame()));
		connect(&optionDialog,SIGNAL(updateFont()),calls,SLOT(setLabelFont()));
		connect(&optionDialog,SIGNAL(updateMenu()),this,SLOT(createMenu()));
		optionDialog.exec();
	}
	else
	{
		OptionsDialog optionDialog(false);
		optionDialog.exec();
	}
}

void TrayShell::help()
{
	QMessageBox::information(0,i18n("TrayImonc -- Help"),i18n("There is no help available at the moment!"));
}

void TrayShell::trafficmonitor(bool theBool)
{
	showTraffic = theBool;
	if(showTraffic && online)
		monitor->show();
	else
		monitor->hide();
}

void TrayShell::isOnline()
{
	if(showTraffic && !online)
	 	monitor->show();
	online = true;
}

void TrayShell::isOffline()
{
	if(showTraffic && online)
		monitor->hide();
	online = false;
}

void TrayShell::createMenu()
{
//	if (menu != NULL)
//		delete(menu);
		
	menu = new Menu(this);
	connect(menu,SIGNAL(setDefaultCircuit(int)),router,SLOT(setDefaultRoute(int)));
	connect(menu,SIGNAL(exitSig()),SLOT(exitApp()));
	connect(menu,SIGNAL(dial()),router,SLOT(default_dial()));
	connect(menu,SIGNAL(addlink()),router,SLOT(addLink()));
	connect(menu,SIGNAL(dialMode(QString)),router,SLOT(setDialMode(QString)));
	connect(menu,SIGNAL(about()),SLOT(about()));
	connect(menu,SIGNAL(options()),SLOT(options()));
	connect(menu,SIGNAL(help()),SLOT(help()));
	connect(menu,SIGNAL(trafficinfo(bool)),SLOT(trafficmonitor(bool)));
	connect(menu,SIGNAL(rebootSig()),router,SLOT(reboot()));
	connect(menu,SIGNAL(haltSig()),router,SLOT(halt()));
	connect(menu,SIGNAL(show_calls(bool)),calls,SLOT(showcalls(bool)));
	connect(menu,SIGNAL(showState()),state,SLOT(showState()));
	connect(tray, SIGNAL(rightclick(QPoint)),menu,SLOT(showSlot(QPoint)));
	connect(router,SIGNAL(addCircuit(QString)),menu,SLOT(addCircuit(QString)));
	connect(router,SIGNAL(setDefaultCircuit(int)),menu,SLOT(setDefaultRoute(int)));
	connect(router,SIGNAL(dialed()),menu,SLOT(dialed()));
	connect(router,SIGNAL(isdndialed()),menu,SLOT(isdnDialed()));
	connect(router, SIGNAL(dialed()),menu,SLOT(dialed()));
	connect(router, SIGNAL(dsloffline()),menu,SLOT(hangup()));
	connect(router, SIGNAL(isdnoffline()),menu,SLOT(isdnHangup()));
	connect(router, SIGNAL(link_added()),menu,SLOT(addedLink()));
	connect(router, SIGNAL(isdntwooffline()),menu,SLOT(removedLink()));
	connect(router, SIGNAL(autodial()),menu,SLOT(dialAuto()));
	connect(router, SIGNAL(manualdial()),menu,SLOT(dialManual()));
	connect(router, SIGNAL(offdial()),menu,SLOT(dialOff()));
	connect(router, SIGNAL(connectedSig(int,QString)),menu,SLOT(connected()));
	connect(router, SIGNAL(noISDN()),menu,SLOT(noISDN()));
	connect(monitor,SIGNAL(closeTheMonitor()),menu,SLOT(traffic_info()));
	
}

