/***************************************************************************
                          calls.cpp  -  description
                             -------------------
    begin                : Wed Oct 24 2001
    copyright            : (C) 2001 by Robert Nitschke,
                           (C) 2003 Sebastian Voitzsch
    email                : bugs@trayimonc.de
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "calls.h"

Calls::Calls()
{
	KSimpleConfig *config = new KSimpleConfig("TrayImonc");
	host = config->readEntry("Server");
	telmondport = config->readUnsignedNumEntry("TelPort");
	if(host == NULL || host == "")
		host = "fli4l";
	if (telmondport == 0)
		telmondport = 5001;
	callTimer = new QTimer(this);	// better set parent so timer will be destroyed automatically
	connect(callTimer,SIGNAL(timeout()),SLOT(callSlot()));
	lastcall = "";
	theLabel = new CallLabel();
	showCall = false;
	callTimer->start(1000, true);
	telmondReConnect = false;
	this->readPhoneBook();
}

Calls::~Calls()
{
//	callTimer->stop();	// no longer neccessary, see above
}

// moved here as telmond connection is always temporary
// so having a fli4ltalk doesn't make sense
QString Calls::getLastCall(QString host_name,int port)
{
	static char callbuf[8192];
	int len;
	int callfd;

	struct sockaddr_in	addr;
	struct hostent *	host_p;
	int opt = 1;
	struct timeval timeout;
	int n;
	fd_set rset;

	(void) memset ((char *) &addr, 0, sizeof (addr));
	
	if ((addr.sin_addr.s_addr = inet_addr (host_name.latin1())) == INADDR_NONE)
	{
		host_p = gethostbyname (host_name.latin1());

		if (! host_p)
		{
			return ((char *) NULL);
		}

		(void) memcpy ((char *) (&addr.sin_addr), host_p->h_addr,host_p->h_length);
	}

	addr.sin_family  = AF_INET;
	addr.sin_port    = htons ((unsigned short) port);
	
	if ((callfd = socket (AF_INET, SOCK_STREAM, 0)) < 0)
	{		
		return ((char *) NULL);
	}

	(void) setsockopt (callfd, IPPROTO_TCP, TCP_NODELAY,(char *) &opt, sizeof (opt));
	
	//set socket to nonblocking
	fcntl (callfd, F_SETFL, O_NONBLOCK);
	n = 0;
	if ((n = (::connect (callfd, (struct sockaddr *) &addr, sizeof (addr)))) < 0)
	{
		if (errno != EINPROGRESS)
		{
			return ((char *) NULL);
		}
	}
	FD_ZERO(&rset);
	FD_SET(callfd, &rset);
	timeout.tv_sec = 3;     // max. 3 seconds for connect
	timeout.tv_usec = 0;

	if(select(callfd+1,&rset,NULL,NULL,&timeout)==0)
	{
		//Connect timeout
		(void) close (callfd);
		return ((char *) NULL);
	}

	fcntl (callfd, F_SETFL, O_RDWR);

	len = read (callfd, callbuf, 8192);

	if (len <= 0)
	{
		(void) close (callfd);
		return ((char *) NULL);
	}

	while (len > 1 && (callbuf[len - 1] == '\n' || callbuf[len - 1] == '\r'))
	{
		callbuf[len - 1] = '\0';
		len--;
	}
	(void) close (callfd);

	return ((char*)callbuf);
}

void Calls::callSlot()
{
	QString answer = this->getLastCall(host,telmondport);
	if(answer == NULL)
	{
		if (!telmondReConnect)
		{
			telmondReConnect=true;		// one more try in case of network errors
			callTimer->start(1000, true);
		}
		return;
	}

	telmondReConnect=false;		// reset retry flag
	if(lastcall == "")
	{
		lastcall = answer;
		
		if(!answer.isEmpty())
		{
			QString date = answer.left(answer.find(" "));
			answer = answer.right(answer.length()-date.length()-1);
			QString time = answer.left(answer.find(" "));
			answer = answer.right(answer.length()-time.length()-1);
			QString source = answer.left(answer.find(" "));
			answer = answer.right(answer.length()-source.length()-1);
			QString aim = answer.left(answer.find("\n"));
		
			QString year = date.left(date.find("/"));
			date = date.right(date.length()-year.length()-1);
			QString month = date.left(date.find("/"));
			date = date.right(date.length()-month.length()-1);
			QString day = date.left(date.find("/"));
			date = "";
			QTextStream ts( &date, IO_WriteOnly );
			ts << day << "." << month << "." << year;
			
			QStringList match = phoneBook.grep(source);
			QString caller = i18n("unknown");
			if (!match.isEmpty())
			{
				caller = match.join("");
				caller = caller.right(caller.length()-caller.findRev(":")-1);
			}

  	
  		emit theLastCall(date,time,caller,source,aim);
  	}
	}
	else if(answer != lastcall)
	{
		lastcall = answer;
		
		QString date = answer.left(answer.find(" "));
		answer = answer.right(answer.length()-date.length()-1);
		QString time = answer.left(answer.find(" "));
		answer = answer.right(answer.length()-time.length()-1);
		QString source = answer.left(answer.find(" "));
		answer = answer.right(answer.length()-source.length()-1);
		QString aim = answer.left(answer.find("\n")-1);
		
		QString year = date.left(date.find("/"));
		date = date.right(date.length()-year.length()-1);
		QString month = date.left(date.find("/"));
		date = date.right(date.length()-month.length()-1);
		QString day = date.left(date.find("/"));
		date = "";
		QTextStream ts( &date, IO_WriteOnly );
		ts << day << "." << month << "." << year;
			
		QStringList match = phoneBook.grep(source);
		QString caller = i18n("unknown");
		if (match.join("") != NULL)
		{
			caller = match.join("");
			caller = caller.right(caller.length()-caller.findRev(":")-1);
		}
	
  	QString out;
  	QTextStream labelText(&out, IO_WriteOnly );
  	if (caller == i18n("unknown"))
		labelText << source << " --> " << aim;
	else
		labelText << caller << " --> " << aim;
  	
  	theLabel->setText(out);
  	theLabel->show();
  	
  	emit newCall(date,time,caller,source,aim);
  }

  callTimer->start(1000, true);
}

void Calls::showcalls(bool set)
{
	showCall = set;
}

void Calls::stop()
{
	callTimer->stop();
}
void Calls::readPhoneBook()
{
	KSimpleConfig *phoneconfig = new KSimpleConfig("TrayImonc"); // we need a new instance here
	phoneBook = phoneconfig->readListEntry("PhoneBook");		// to update phonebook changes properly
}

void Calls::setLabelFont()
{
	theLabel->setFont();
}
